import { PrismaClientKnownRequestError } from '@prisma/client/runtime';
import { Request, Response, Router } from 'express';
import { adminAuthorizationMiddleware } from '../middlewares/admin_authorization.middleware';
import { authenticationMiddleware } from '../middlewares/authentication.middleware';
import { AppError } from '../shared/AppError';
import { prisma } from '../utils/prisma';

const ConditionsRouter = Router();

interface IConditionRequestBody {
  abbr: string;
  description: string;
}

type IConditionUpdateRequestBody = IConditionRequestBody & { deletedAt: string | null };

/*Lista todas as condições de saúde existentes no banco de dados e envia no corpo da resposta*/
ConditionsRouter.get("/", async (request: Request, response: Response): Promise<Response> => {
  /*Faz a busca no banco pelas condições*/
  const listConditions = await prisma.conditions.findMany({
    orderBy: {
      createdAt: 'asc',
    },
  });

  /*Retorna as condições*/
  return response.status(200).json(listConditions);
});

/*Adiciona uma nova condição de saúde no banco de dados, isso se ela não existir (a sigla tem que ser diferente)*/
ConditionsRouter.post(
  "/",
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {

    const newCondition: IConditionRequestBody = request.body;

    try {
      /*Faz a inserção da nova condição de saúde*/
      const condition = await prisma.conditions.create({
        data: {
          abbr: newCondition.abbr.toUpperCase(),
          description: newCondition.description
        }
      });

      return response.status(202).json(condition);
    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === 'P2002') {
          return new AppError(400, "Condição de saúde já existente no sistema").execute(response);
        }
      }

      return new AppError(400, "Condição de saúde não pode ser salva no sistema").execute(response);
    }
  }
);

/*Altera uma condição de saúde já existente no sistema*/
ConditionsRouter.put(
  "/:id",
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const conditionRequest: IConditionUpdateRequestBody = request.body;
    const { id } = request.params;

    try {
      const condition = await prisma.conditions.update({
        where: {
          id,
        },
        data: {
          abbr: conditionRequest.abbr.toUpperCase(),
          description: conditionRequest.description,
          updatedAt: new Date().toISOString(),
          deletedAt: conditionRequest.deletedAt,
        }
      });

      return response.status(202).json(condition);

    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === 'P2002') {
          return new AppError(400, "Condição de saúde já existente no sistema").execute(response);
        }

        if (error.code === 'P2025') {
          return new AppError(404, "Condição de saúde não encontrada").execute(response);
        }
      }
      return new AppError(400, "Condição de saúde não pode ser modificada").execute(response);
    }
  }
);

/*Deleção lógica de uma condição de saúde*/
ConditionsRouter.delete(
  "/:id",
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const { id } = request.params;

    try {

      await prisma.conditions.update({
        where: {
          id,
        },
        data: {
          deletedAt: new Date()
        },
      });

      return response.status(200).json({ message: "Condição de saúde desativada!" });
    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === 'P2025') {
          return new AppError(404, "Condição de saúde não encontrada").execute(response);
        }
      }
      return new AppError(400, "Não foi possível desativar a condição de saúde!").execute(response);
    }
  }
);

/*Busca uma condição de saúde pelo id da mesma*/
ConditionsRouter.get("/:id", async (request: Request, response: Response): Promise<Response> => {
  const { id } = request.params;

  try {
    const condition = await prisma.conditions.findUnique({
      where: {
        id,
      }
    });

    if (!condition) {
      return new AppError(404, "Condição de saúde não encontrada").execute(response);
    }

    return response.status(200).json(condition);
  } catch (error) {

    return new AppError(400, "Erro ao buscar a condição de saúde").execute(response);
  }
});

export { ConditionsRouter }

