import { PrismaClientKnownRequestError } from '@prisma/client/runtime';
import { Request, Response, Router } from 'express';
import { adminAuthorizationMiddleware } from '../middlewares/admin_authorization.middleware';
import { authenticationMiddleware } from '../middlewares/authentication.middleware';
import { AppError } from '../shared/AppError';
import { prisma } from '../utils/prisma';

const MicroareasRouter = Router();

interface Coords {
  lat: number;
  lng: number;
}

interface IMicroareaRequestBody {
  number: number;
  localization: Coords[];
}

/* Lista todas as microáeras existentes no banco de dados e envia no corpo da resposta */
MicroareasRouter.get('/', async (request: Request, response: Response): Promise<Response> => {
  /*Faz a busca no banco pelas microáreas*/
  const listMicroareas = await prisma.microareas.findMany({
    orderBy: {
      createdAt: 'asc',
    },
    select: {
      id: true,
      number: true,
      localization: {},
      createdAt: true,
      deletedAt: true,
    },

  });

  /*Retorna as condições*/
  return response.status(200).json(listMicroareas);
});

/*Adiciona uma nova microárea no banco de dados, isso se ela não existir (o número tem que ser diferente)*/
MicroareasRouter.post(
  '/',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const newMicroarea: IMicroareaRequestBody = request.body;
    const { number, localization } = newMicroarea;

    if (localization.length <= 0) {
      return new AppError(400, "Microárea não pode ser salva no sistema, falta as coordenadas.").execute(response);
    }

    const localizationData = localization.map((coord) => {
      return {
        latitude: coord.lat,
        longitude: coord.lng,
      }
    });

    try {

      /*Faz a inserção da nova condição de saúde*/
      const microarea = await prisma.microareas.create({
        data: {
          number,
          localization: {
            createMany: {
              data: localizationData
            }
          }
        },
      });

      return response.status(202).json(microarea);
    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === 'P2002') {
          return new AppError(400, "Microárea já existente no sistema").execute(response);
        }
      }

      return new AppError(400, "Microárea não pode ser salva no sistema").execute(response);
    }
  }
);

/* Altera uma microárea já existente no sistema*/
MicroareasRouter.put(
  "/:id",
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const { id } = request.params;
    const microareaData: IMicroareaRequestBody = request.body;
    const { number, localization } = microareaData;

    try {
      const microareaFound = await prisma.microareas.findUnique({
        where: {
          id,
        },
        include: {
          localization: true,
        }
      });

      if (!microareaFound) {
        return new AppError(404, "Microárea não encontrada no sistema").execute(response);
      }

      try {

        const microarea = await prisma.microareas.update({
          where: {
            id: microareaFound.id,
          },
          data: {
            number,
            localization: {
              deleteMany: {
                microareaId: microareaFound.id,
              },
              create: localization.map((coord) => {
                return {
                  latitude: coord.lat,
                  longitude: coord.lng,
                }
              })
            }
          }
        });

        return response.status(200).json(microarea);

      } catch (error) {
        if (error instanceof PrismaClientKnownRequestError) {
          if (error.code === 'P2002') {
            return new AppError(400, "Microárea já existente no sistema").execute(response);
          }
        }

        return new AppError(400, "Microárea não pode ser alterada no sistema").execute(response);
      }

    } catch (error) {
      return new AppError(404, "Microárea não pode ser localizada no sistema").execute(response);
    }
  }
);

/* TODO - Deleção lógicaa de uma microárea */
// MicroareasRouter.delete(
//   "/:id",
//   authenticationMiddleware,
//   adminAuthorizationMiddleware,
//   async (request: Request, response: Response): Promise<Response> => {
//     return response.json({});
//   }
// );

/*Busca uma microárea pelo id da mesma*/
MicroareasRouter.get("/:id", async (request: Request, response: Response): Promise<Response> => {
  const { id } = request.params;

  try {
    const microarea = await prisma.microareas.findUnique({
      where: {
        id,
      },
      select: {
        id: true,
        number: true,
        localization: {
          select: {
            latitude: true,
            longitude: true,
          }
        },
      }
    });

    if (!microarea) {
      return new AppError(404, "Microárea não encontrada").execute(response);
    }

    return response.status(200).json(microarea);
  } catch (error) {

    return new AppError(400, "Erro ao buscar a microárea").execute(response);
  }
});

export { MicroareasRouter };