import { PrismaClientKnownRequestError } from '@prisma/client/runtime';
import { create } from 'domain';
import { NextFunction, Request, Response, Router } from 'express';
import { adminAuthorizationMiddleware } from '../middlewares/admin_authorization.middleware';
import { authenticationMiddleware } from '../middlewares/authentication.middleware';
import { AppError } from '../shared/AppError';
import { prisma } from '../utils/prisma';

const PatientsRouter = Router();

interface IPatientsCreateRequestBody {
  cpf: string;
  name: string;
  observation?: string;
  sex: 'FEMININO' | 'MASCULINO';
  birthdate: string;
  address: {
    publicPlace: string; /*Rua/Logadouro*/
    number: number;
    complement?: string;
    district: string;   /*Bairro/Distrito*/
  },
  microareaId: string;
  localization: {
    lat: number;
    lng: number;
  },
  conditions: string[];
}

/* Criação de um novo paciente */
PatientsRouter.post(
  '/',
  authenticationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const data: IPatientsCreateRequestBody = request.body;
    const userId = request.user.id;

    try {
      const patient = await prisma.patients.create({
        data: {
          cpf: data.cpf,
          name: data.name,
          sex: data.sex,
          birthdate: new Date(data.birthdate),
          address: {
            create: {
              publicPlace: data.address.publicPlace,
              district: data.address.district,
              complement: data.address.complement,
              number: data.address.number
            }
          },
          localization: {
            create: {
              latitude: data.localization.lat,
              longitude: data.localization.lng,
            }
          },
          microareaId: data.microareaId,
          userId: userId,
          conditions: {
            create: data.conditions.map(condition => {
              return {
                conditionId: condition
              }
            }),
          }
        }
      });

      return response.status(202).json(patient);

    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === 'P2002') {
          return new AppError(400, "Paciente já cadastrado no sistema!").execute(response);
        }
      }

      console.log(error);

      return new AppError(400, "Paciente não pode ser criado no momento!").execute(response);
    }
  }
);

const selectPatients = {
  id: true,
  name: true,
  cpf: true,
  birthdate: true,
  sex: true,
  Microareas: {
    select: {
      number: true,
    }
  },
  localization: {
    select: {
      latitude: true,
      longitude: true,
    }
  },
  conditions: {
    select: {
      condition: {
        select: {
          id: true,
          abbr: true,
        }
      },
    }
  }
}

const patientDataFormatted = function (patients: any) {
  return patients.map((patient: any) => {
    return {
      id: patient.id,
      name: patient.name,
      cpf: patient.cpf,
      birthdate: patient.birthdate,
      sex: patient.sex,
      microarea: patient.Microareas.number,
      localization: patient.localization[0],
      conditions: patient.conditions.map((condition: any) => {
        return {
          id: condition.condition?.id,
          abbr: condition.condition?.abbr,
        }
      })
    }
  });
}

/* Listar pacientes */
PatientsRouter.get(
  '/',
  authenticationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    /* Listar todos os pacientes */
    if (request.user.role === 'ADMIN') {
      const patients = await prisma.patients.findMany({
        select: selectPatients,
      });

      const data = patientDataFormatted(patients);

      return response.status(200).json(data);
    }

    /* Listar pacientes para os Agentes */
    const patients = await prisma.patients.findMany({
      select: selectPatients,
      where: {
        userId: request.user.id,
      }
    });

    const data = patientDataFormatted(patients);

    return response.status(200).json(data);
  }
);

/* Busca paciente pelo ID*/
PatientsRouter.get(
  '/:id',
  authenticationMiddleware,
  async (request: Request, response: Response, next: NextFunction): Promise<Response | void> => {
    const { id } = request.params;

    if (Object.keys(request.query).length !== 0) {
      return next();
    }

    try {
      const patient = await prisma.patients.findUnique({
        where: {
          id,
        },
        select: {
          id: true,
          name: true,
          cpf: true,
          birthdate: true,
          sex: true,
          observation: true,
          address: {
            select: {
              number: true,
              district: true,
              complement: true,
              publicPlace: true,
            }
          },
          Microareas: {
            select: {
              id: true,
            }
          },
          localization: {
            select: {
              latitude: true,
              longitude: true,
            }
          },
          conditions: {
            select: {
              condition: {
                select: {
                  id: true,
                  abbr: true,
                }
              },
            }
          }
        }
      });

      if (!patient) {
        return new AppError(404, "Paciente não encontrado!").execute(response);
      }
      const data = {
        ...patient,
        microareaId: patient.Microareas.id,
        localization: patient.localization[0],
        conditions: patient.conditions.map(condition => {
          return {
            id: condition.condition?.id,
            abbr: condition.condition?.abbr,
          }
        }),
        Microareas: undefined,
        address: patient.address[0],
      }

      delete data.Microareas;

      return response.status(200).json(data);

    } catch (error) {
      return new AppError(400, "Não foi possível achar o paciente!").execute(response);
    }
  }
);

const getConditionToInsert = function (data: any, patient: any) {
  return data.conditions.filter((condition: string) => isValidConditionToInsert(condition, patient)).map((condition: any) => {
    return {
      conditionId: condition
    }
  });
}

const isValidConditionToInsert = function (condition: string, patient: { conditions: { conditionId: string }[] }): boolean {
  return patient.conditions.filter((contidionOld) => contidionOld.conditionId === condition).length > 0 ? false : true;
}

/* Atualiza dados do paciente */
PatientsRouter.put(
  '/:id',
  authenticationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const { id } = request.params;
    const data: IPatientsCreateRequestBody = request.body;


    try {
      const patientFound = await prisma.patients.findUnique({
        where: {
          id,
        },
        include: {
          address: true,
          conditions: true,
          localization: true,
        }
      });
      if (!patientFound) {
        return new AppError(404, "Paciente não encontrado!").execute(response);
      }

      try {
        const patient = await prisma.patients.update({
          where: {
            id,
          },
          data: {
            cpf: data.cpf,
            name: data.name,
            sex: data.sex,
            birthdate: new Date(data.birthdate),
            observation: data.observation,
            address: {
              update: {
                where: {
                  id: patientFound?.address[0].id
                },
                data: {
                  publicPlace: data.address.publicPlace,
                  district: data.address.district,
                  complement: data.address.complement,
                  number: data.address.number
                }
              }
            },
            localization: {
              update: {
                where: {
                  id: patientFound?.localization[0].id,
                },
                data: {
                  latitude: data.localization.lat,
                  longitude: data.localization.lng,
                }
              }
            },
            microareaId: data.microareaId,
            updatedAt: new Date().toISOString(),
            conditions: {
              create: getConditionToInsert(data, patientFound),
              deleteMany: patientFound.conditions.filter(condition => {
                return !data.conditions.includes(condition.conditionId)
              }),
            }
          },
          include: {
            conditions: true
          }
        });

        return response.status(200).json(patient);

      } catch (error) {
        return new AppError(400, "Não foi possível atualizar o paciente!").execute(response);
      }

    } catch (error) {
      return new AppError(400, "Não foi possível achar o paciente!").execute(response);
    }
  }
);

/* TODO - Busca Personalizada de Pacientes */
interface PatientsSearchQueryParams {
  user?: string;
  microarea?: string;
  under_five_years_old?: string;
  conditions?: string;
}

PatientsRouter.get(
  '/search',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {

    const { conditions, microarea, user, under_five_years_old } = request.query as PatientsSearchQueryParams;
    const selectPatients = {
      id: true,
      name: true,
      cpf: true,
      birthdate: true,
      sex: true,
      observation: true,
      address: {
        select: {
          number: true,
          district: true,
          complement: true,
          publicPlace: true,
        }
      },
      Microareas: {
        select: {
          number: true,
        }
      },
      localization: {
        select: {
          latitude: true,
          longitude: true,
        }
      },
      conditions: {
        select: {
          condition: {
            select: {
              id: true,
              abbr: true,
            }
          },
        }
      }
    }

    const conditionsArray = conditions ? conditions?.split(',') : [];

    try {

      const patients = await prisma.patients.findMany({
        where: {
          ...(user && {
            userId: user
          }),
          ...(microarea && Number(microarea) && {
            Microareas: {
              number: Number(microarea)
            }
          }),
          ...(under_five_years_old && under_five_years_old === 'true' && {
            birthdate: {
              gte: new Date(new Date().getFullYear() - 5, new Date().getMonth(), new Date().getDate())
            }
          }),
          ...(conditionsArray && conditionsArray.length > 0 && {
            conditions: {
              some: {
                OR: conditionsArray.map((condition) => {
                  return {
                    conditionId: condition
                  }
                })
              }
            }
          }),
        },
        select: selectPatients,
      });


      const patientsFormatted = patientDataFormatted(patients);
      return response.status(200).json(patientsFormatted);

    } catch (error) {
      return new AppError(400, "Não foi possível achar os pacientes!").execute(response);
    }
  }
);


export { PatientsRouter };