import { Request, Response, Router } from 'express';
import { AppError } from '../shared/AppError';
import { encryptedPassword, verifyPassword } from '../utils/bcrypt';
import jwt from 'jsonwebtoken';
import { authenticationMiddleware } from '../middlewares/authentication.middleware';
import { adminAuthorizationMiddleware } from '../middlewares/admin_authorization.middleware';
import { prisma } from '../utils/prisma';

interface IUserRequestBody {
  name: string,
  cpf: string,
  password: string,
}

const UsersRouter = Router();

// Rota de login da aplicação
UsersRouter.post('/login', async (request: Request, response: Response): Promise<Response> => {
  const { cpf, password } = request.body;

  const user = await prisma.user.findUnique({
    where: {
      cpf,
    }
  });

  // se o usuário não foi encontrado
  if (!user) {
    return new AppError(400, "CPF ou senha estão incorretos!").execute(response);
  }

  if (user.deletedAt) {
    return new AppError(400, "Usuário desativado!").execute(response);
  }

  const checkPassword = verifyPassword(password, user.password);
  // verifica se a senha está correta
  if (!checkPassword) {
    return new AppError(400, "CPF ou senha estão incorretos!").execute(response);
  }

  const userInfo = {
    id: user.id,
    role: user.role,
    name: user.name
  };

  // Cria token de autenticação
  const token = jwt.sign({}, process.env.JWT_SECRET, {
    subject: user.id,
    expiresIn: "1d",
  });

  return response.status(200).json({
    user: userInfo,
    token
  });
});

UsersRouter.post(
  '/agents',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const data: IUserRequestBody = request.body;

    try {
      const newUser = await prisma.user.create({
        data: {
          cpf: data.cpf,
          name: data.name,
          password: encryptedPassword(data.password),
        },
        select: {
          id: true,
          name: true,
          cpf: true,
          role: true,
          createdAt: true,
          deletedAt: true
        }
      });

      return response.status(202).json(newUser);
    } catch {
      return new AppError(400, "Usuário já existente no sistema").execute(response);
    }
  }
);

UsersRouter.get(
  '/agents',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const listUsersAgents = await prisma.user.findMany({
      where: {
        role: 'AGENT'
      },
      orderBy: {
        createdAt: 'asc',
      },
      select: {
        id: true,
        name: true,
        cpf: true,
        role: true,
        createdAt: true,
        deletedAt: true
      }
    });

    return response.status(200).json(listUsersAgents);
  }
);

UsersRouter.delete(
  '/agents/:id',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const { id } = request.params;
    try {

      await prisma.user.update({
        where: {
          id,
        },
        data: {
          deletedAt: new Date()
        },
      });

      return response.status(200).json({ message: "Usuário desativado com sucesso!" });
    } catch {
      return new AppError(400, "Não foi possível deletar o usuário!").execute(response);
    }
  }
);

UsersRouter.get(
  '/agents/:id',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const { id } = request.params;
    try {
      const userAgent = await prisma.user.findUnique({
        where: {
          id: id
        },
        select: {
          id: true,
          name: true,
          cpf: true,
          role: true,
          createdAt: true,
          deletedAt: true
        }
      });

      return response.status(200).json(userAgent);
    } catch {
      return new AppError(400, "Não foi possível encontrar o usuário!").execute(response);
    }

  }
);


type IUserUpdateRequestBody = IUserRequestBody & { deletedAt: string | null };
UsersRouter.put(
  '/agents/:id',
  authenticationMiddleware,
  adminAuthorizationMiddleware,
  async (request: Request, response: Response): Promise<Response> => {
    const data: IUserUpdateRequestBody = request.body;
    const { id } = request.params;

    try {
      const updatedUser = await prisma.user.update({
        where: {
          id,
        },
        data: {
          cpf: data.cpf,
          name: data.name,
          password: encryptedPassword(data.password),
          deletedAt: data.deletedAt
        },
        select: {
          id: true,
          name: true,
          cpf: true,
          role: true,
          createdAt: true,
          deletedAt: true
        }
      });

      return response.status(202).json(updatedUser);
    } catch {
      return new AppError(400, "Não foi possível atualizar o usuário").execute(response);
    }
  }
);


export { UsersRouter };