import { createContext, ReactElement, useContext, useEffect, useState } from "react";
import { api } from "../api/axios";
import { HeadersDefaults } from 'axios';

interface AuthProviderProps {
  children: ReactElement;
}

interface UserData {
  id: string;
  role: 'ADMIN' | 'AGENT';
  name: string;
}

interface AuthContextData {
  signed: boolean;
  user: UserData | null;
  signIn: (user: UserData, token: string) => void;
  signOut: () => void;
}

interface CommonHeaderProperties extends HeadersDefaults {
  Authorization: string;
}

const AuthContext = createContext<AuthContextData>({} as AuthContextData);

function AuthProvider({ children }: AuthProviderProps) {
  const [user, setUser] = useState<UserData | null>(() => {
    const storagedUser = localStorage.getItem('@WebAuth:user');
    const storagedToken = localStorage.getItem('@WebAuth:token');

    if (storagedUser && storagedToken) {
      if (!isValidToken(storagedToken)) {
        return null;
      } else {
        const user = JSON.parse(storagedUser);

        api.defaults.headers = {
          Authorization: `Bearer ${storagedToken}`
        } as CommonHeaderProperties;
        return user
      }
    } else {
      return null;
    }
  });


  function isValidToken(token: string) {
    try {
      const decodedToken = JSON.parse(atob(token.split(".")[1]));

      if (decodedToken.exp * 1000 < Date.now()) {
        return false
      }
      return true;
    } catch (e) {
      return null;
    }
  }

  useEffect(() => {
    const storagedUser = localStorage.getItem('@WebAuth:user');
    const storagedToken = localStorage.getItem('@WebAuth:token');

    if (storagedUser && storagedToken) {
      if (!isValidToken(storagedToken)) {
        signOut();
      } else {
        setUser(JSON.parse(storagedUser))

        api.defaults.headers = {
          Authorization: `Bearer ${storagedToken}`
        } as CommonHeaderProperties;
      }
    } else {
      setUser(null);
    }
  }, []);

  const signIn = (user: UserData, token: string) => {
    setUser(user);

    api.defaults.headers = {
      Authorization: `Bearer ${token}`
    } as CommonHeaderProperties;

    localStorage.setItem('@WebAuth:user', JSON.stringify(user));
    localStorage.setItem('@WebAuth:token', token);
  }

  const signOut = () => {
    setUser(null);
    localStorage.clear();
  }

  return (
    <AuthContext.Provider value={{ signed: !!user, user, signIn, signOut }}>
      {children}
    </AuthContext.Provider>
  );
}

function useAuth() {
  const context = useContext(AuthContext);
  return context;
}

export { useAuth, AuthProvider }