import { useCallback, useState } from "react";
import { Input } from "../../../components/Input";
import { Sidebar } from "../../../components/Sidebar";
import * as yup from 'yup';
import { api } from "../../../api/axios";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";
import { useNavigate } from "react-router-dom";

const schema = yup.object().shape({
  name: yup.string()
    .required("Nome é obrigatório"),
  cpf: yup.string()
    .required("CPF é obrigatório")
    .max(14, "Preencha o CPF corretamente")
    .min(13, "Preencha o CPF corretamente"),
  password: yup.string()
    .required("Senha é obrigatório")
    .min(8, "Senha curta - deve ter no mínimo 8 caracteres"),
});

interface AgentForm {
  name: string;
  cpf: string;
  password: string;
}

interface AgentFormErrors {
  name: string | null;
  cpf: string | null;
  password: string | null;
}

function CreateAgents() {
  const navigate = useNavigate();
  const [isOpen, setIsOpen] = useState(false);
  const [agent, setAgent] = useState<AgentForm>({} as AgentForm);
  const [agentErrors, setAgentErrors] = useState<AgentFormErrors>({
    cpf: null,
    name: null,
    password: null
  } as AgentFormErrors);

  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement>) => {
    setAgent({
      ...agent,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setAgentErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [agent]);


  const handleCancel = () => {
    navigate('/dashboard');
  }

  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();

    try {
      const dataValidate = await schema.validate({
        name: agent.name,
        cpf: agent.cpf,
        password: agent.password
      }, {
        abortEarly: false
      });

      try {
        const cpfWithoutFormatting = dataValidate.cpf.replace(/(\.|\-)/g, '');
        const response = await api.post('/users/agents', {
          ...dataValidate,
          cpf: cpfWithoutFormatting
        });

        if (response.status === 202) {
          navigate('/agents/list');
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setAgentErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }
        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Adicione um Agente</h2>
        </div>
        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Nome Completo"
                id="name"
                type="text"
                name="name"
                placeholder="Seu nome completo"
                error={agentErrors.name ? agentErrors.name : ''}
                onChange={handleChange}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="CPF"
                id="cpf"
                type="text"
                name="cpf"
                mask="CPF"
                placeholder="999.999.999-99"
                onChange={handleChange}
                error={agentErrors.cpf ? agentErrors.cpf : ''}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Password"
                name="password"
                id="password"
                placeholder="******************"
                type="password"
                onChange={handleChange}
                error={agentErrors.password ? agentErrors.password : ''}
              />
            </div>
          </div>


          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { CreateAgents };