import { useCallback, useEffect, useState } from "react";
import { Input } from "../../../components/Input";
import { Sidebar } from "../../../components/Sidebar";
import * as yup from 'yup';
import { api } from "../../../api/axios";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";
import { useNavigate, useParams } from "react-router-dom";
import { CPFMaskToView } from "../../../utils/masks";

const schema = yup.object().shape({
  name: yup.string()
    .required("Nome é obrigatório"),
  cpf: yup.string()
    .required("CPF é obrigatório")
    .max(11, "Preencha o CPF corretamente")
    .min(11, "Preencha o CPF corretamente"),
  password: yup.string()
    .required("Senha é obrigatório")
    .min(5, "Senha curta - deve ter no mínimo 5 caracteres"),
});

interface AgentResponse {
  id: string,
  name: string,
  cpf: string,
  deletedAt: string | null;
}

interface AgentForm {
  id: string,
  name: string;
  cpf: string;
  password: string;
  deletedAt: string | null;
}

interface AgentFormErrors {
  name: string | null;
  cpf: string | null;
  password: string | null;
}

function UpdateAgents() {
  const navigate = useNavigate();
  const { id } = useParams();
  const [isOpen, setIsOpen] = useState(false);
  const [agent, setAgent] = useState<AgentForm>({
    cpf: "",
    name: "",
    password: "",
  } as AgentForm);
  const [activeAgent, setActiveAgent] = useState<boolean>(false);
  const [agentErrors, setAgentErrors] = useState<AgentFormErrors>({
    cpf: null,
    name: null,
    password: null
  } as AgentFormErrors);

  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }


  useEffect(() => {
    /*Buscar Agente*/
    const getAgent = async () => {
      try {
        const response = await api.get<AgentResponse>(`/users/agents/${id}`);

        if (response.data) {
          const updateAgent = {
            id: response.data.id,
            password: '',
            cpf: response.data.cpf,
            name: response.data.name,
            deletedAt: response.data.deletedAt
          } as AgentForm;

          setAgent(updateAgent);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }

    getAgent();
  }, []);

  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement>) => {
    setAgent({
      ...agent,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setAgentErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [agent]);


  const handleCancel = () => {
    navigate('/dashboard');
  }

  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();

    try {
      const dataValidate = await schema.validate({
        name: agent.name,
        cpf: agent.cpf.replace(/(\.|\-)/g, ''),
        password: agent.password
      }, {
        abortEarly: false
      });

      try {
        const cpfWithoutFormatting = dataValidate.cpf.replace(/(\.|\-)/g, '');

        const response = await api.put(`/users/agents/${agent.id}`, {
          ...dataValidate,
          cpf: cpfWithoutFormatting,
          deletedAt: activeAgent ? null : agent.deletedAt
        });

        if (response.data) {
          toast.success("Usuário atualizado com sucesso", {
            duration: 4000,
            position: 'top-center'
          });
        }

      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setAgentErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }
        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Editando {agent.name ? agent.name : ''}</h2>
        </div>
        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Nome Completo"
                id="name"
                type="text"
                name="name"
                placeholder="Seu nome completo"
                error={agentErrors.name ? agentErrors.name : ''}
                onChange={handleChange}
                value={agent.name}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="CPF"
                id="cpf"
                type="text"
                name="cpf"
                mask="CPF"
                placeholder="999.999.999-99"
                onChange={handleChange}
                error={agentErrors.cpf ? agentErrors.cpf : ''}
                value={agent.cpf ? CPFMaskToView(agent.cpf) : ''}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Password"
                name="password"
                id="password"
                placeholder="******************"
                type="password"
                onChange={handleChange}
                error={agentErrors.password ? agentErrors.password : ''}
                value={agent.password}
              />
            </div>
            {agent?.deletedAt &&
              <div className="px-3 mt-6 flex items-center justify-start w-full">
                <input
                  id="active"
                  name="active"
                  type="checkbox"
                  checked={activeAgent}
                  onChange={(e) => setActiveAgent(e.target.checked)}
                  className="h-4 w-4  bg-cyan-500 border border-gray-300 focus:border-cyan-300 focus:outline-none focus:ring focus:ring-cyan-200 focus:ring-opacity-50 rounded-md shadow-sm disabled:bg-gray-100" />
                <label htmlFor="active" className="ml-2 block text-sm text-gray-900">
                  Ativar agente
                </label>
              </div>
            }
          </div>


          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { UpdateAgents };