import React, { FormEvent, useCallback, useState } from "react";
import { useNavigate } from "react-router-dom";
import * as yup from 'yup';
import { api } from "../../../api/axios";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";

import { Input } from "../../../components/Input";
import { Sidebar } from "../../../components/Sidebar";
import { Textarea } from "../../../components/Textarea";

interface ConditionsForm {
  abbr: string;
  description: string;
}

interface ConditionsFormErrors {
  abbr: string | null;
  description: string | null;
}

const schema = yup.object().shape({
  abbr: yup.string()
    .required("Sigla é obrigatória")
    .max(10, "Máximo de 10 caracteres"),
  description: yup.string()
    .required("Descrição é obrigatório"),
});

function CreateConditions() {
  const navigate = useNavigate();
  const [isOpen, setIsOpen] = useState(false);
  const [condition, setCondition] = useState<ConditionsForm>({} as ConditionsForm);
  const [conditionErrors, setConditionErrors] = useState<ConditionsFormErrors>({
    abbr: null,
    description: null,
  } as ConditionsFormErrors);

  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement> | React.FormEvent<HTMLTextAreaElement>) => {
    setCondition({
      ...condition,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setConditionErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [condition]);

  const handleSubmit = async (event: FormEvent) => {
    event.preventDefault();

    try {
      const dataValidate = await schema.validate({
        abbr: condition.abbr,
        description: condition.description,
      }, {
        abortEarly: false
      });

      try {
        const response = await api.post('/conditions', {
          ...dataValidate,
        });

        if (response.status === 202) {
          navigate('/conditions/list');
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setConditionErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  const handleCancel = () => {
    navigate('/dashboard');
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }

        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Criar condição de saúde</h2>
        </div>

        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Condição de saúde"
                id="abbr"
                type="text"
                name="abbr"
                mask="ABBR"
                placeholder="Sigla da condição de saúde"
                error={conditionErrors.abbr ? conditionErrors.abbr : ''}
                onChange={handleChange}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Textarea
                label="Descrição"
                id="description"
                name="description"
                placeholder="Digite a descrição para a condição de saúde"
                error={conditionErrors.description ? conditionErrors.description : ''}
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { CreateConditions }