import { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";
import ReactPaginate from "react-paginate";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";
import { CaretLeft, CaretRight, DotsThree } from "phosphor-react";


import { api } from "../../../api/axios";
import { Sidebar } from "../../../components/Sidebar";
import { useAuth } from "../../../context/AuthContext";
import { ModalConfirmation } from "../../../components/Modal";

interface ConditionResponse {
  id: string;
  abbr: string;
  description: string;
  deletedAt: string | null;
}

interface ConditionDeleteResponse {
  message: string;
}

function ListConditions() {
  const { user } = useAuth();
  const navigate = useNavigate();
  const [isOpen, setIsOpen] = useState(false);
  const [conditions, setConditions] = useState<ConditionResponse[]>([]);
  const [showModalDelete, setShowModalDelete] = useState(false);
  const [conditionToDelete, setConditionToDelete] = useState<ConditionResponse | null>(null);

  /*Para a paginação*/
  const [pageCount, setPageCount] = useState(0);
  const [itemOffset, setItemOffset] = useState(0);
  const [currentItems, setCurrentItems] = useState<ConditionResponse[] | null>(null);
  const itemsPerPage = 10;

  useEffect(() => {
    if (conditions) {
      const endOffset = itemOffset + itemsPerPage;

      setCurrentItems(conditions.slice(itemOffset, endOffset));
      setPageCount(Math.ceil(conditions.length / itemsPerPage));
    }
  }, [itemOffset, itemsPerPage, conditions]);

  useEffect(() => {
    const listConditons = async () => {
      try {
        const response = await api.get<ConditionResponse[]>('/conditions');

        if (response.data) {
          setConditions(response.data);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listConditons();
  }, [user]);

  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  const handleDeleteAgent = async (conditionId: string | undefined) => {
    /*Apenas desativar o Agente*/
    if (!conditionId) {
      setConditionToDelete(null);
      setShowModalDelete(false);
      return;
    }

    try {
      const response = await api.delete<ConditionDeleteResponse>(`/conditions/${conditionId}`);

      if (response.data) {
        setConditionToDelete(null);
        setShowModalDelete(false);

        const updateConditions = conditions.map(condition => {
          if (condition.id === conditionId) {
            return {
              ...condition,
              deletedAt: new Date().toISOString()
            }
          } else {
            return condition;
          }
        });
        setConditions(updateConditions);

        toast.success(response.data.message, {
          duration: 4000,
          position: 'top-center'
        });
      }

    } catch (error) {
      if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
        if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
          toast.error(error.response.data.message, {
            duration: 4000,
            position: 'top-center'
          })
        }
      }
    }
  }

  const handleCancelDelete = () => {
    /*Cancelar a ação de deletar e fechar o modal*/
    setConditionToDelete(null);
    setShowModalDelete(false);
  }

  const handleModifyCondition = (conditionId: string) => {
    /*Mandar para a tela de alterar agente*/
    navigate(`/conditions/update/${conditionId}`);
  }

  const handlePageClick = (event: { selected: number }) => {
    const newOffset = event.selected * itemsPerPage % conditions.length;
    setItemOffset(newOffset);
  };

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>

      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }

        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Lista de condições de saúde</h2>
        </div>

        <div className="overflow-x-auto relative w-full px-5 lg:px-8 bg-gray-50 py-6 lg:max-w-[90%] mx-auto mb-5">
          <table className="w-full text-sm text-left text-gray-500 dark:text-gray-400">
            <thead className="sticky top-0 text-xs text-gray-700 uppercase bg-gray-50 dark:bg-gray-700 dark:text-gray-400">
              <tr>
                <th scope="col" className="py-3 px-6">
                  Sigla
                </th>
                <th scope="col" className="py-3 px-6 max-w-[200px]">
                  Descrição
                </th>
                <th scope="col" className="py-3 px-6">
                  Status
                </th>
                <th scope="col" className="py-3 px-6">
                  Ações
                </th>
              </tr>
            </thead>
            <tbody className="overflow-x-auto ">
              {currentItems?.map((condition) => {
                return (
                  <tr key={condition.id} className="bg-white border-b dark:bg-gray-800 dark:border-gray-700 hover:bg-gray-50 dark:hover:bg-gray-600">
                    <td scope="row" className="py-4 px-6 font-medium text-gray-900 whitespace-nowrap dark:text-white">
                      {condition.abbr}
                    </td>
                    <td className="py-4 px-6 max-w-[200px]">
                      {condition.description}
                    </td>
                    <td className="py-4 px-6">
                      {condition.deletedAt === null ? 'Ativo' : 'Inativo'}
                    </td>
                    <td className="py-4 px-6">
                      <div className="w-full px-3 flex mm:space-x-2 flex-wrap space-y-2 mm:space-y-0">
                        <button className="btn_error text-white w-full mm:w-auto" onClick={() => { setShowModalDelete(true); setConditionToDelete(condition) }}>Desativar</button>
                        <button className="btn_success w-full mm:w-auto" onClick={() => handleModifyCondition(condition.id)}>Alterar</button>
                      </div>
                    </td>
                  </tr>
                );
              })}
            </tbody>
          </table>

          {conditions.length > itemsPerPage && (
            <nav className="flex justify-between items-center pt-4" aria-label="Table navigation">
              <span className="text-sm font-normal text-gray-500 dark:text-gray-400">Listando <span className="font-semibold text-gray-900 dark:text-white">{itemOffset === 0 ? 1 : itemOffset + 1} - {(itemsPerPage + itemOffset) > conditions.length ? conditions.length : (itemsPerPage + itemOffset)}</span> de <span className="font-semibold text-gray-900 dark:text-white">{conditions.length}</span></span>
              <div className="inline-flex items-center -space-x-px">
                <ReactPaginate
                  pageCount={pageCount}
                  onPageChange={handlePageClick}
                  pageRangeDisplayed={2}
                  marginPagesDisplayed={1}
                  nextLabel={<CaretRight size={20} />}
                  previousLabel={<CaretLeft size={20} />}
                  breakLabel={<DotsThree size={32} />}
                  containerClassName="inline-flex"
                  activeClassName="bg-indigo-600"
                  activeLinkClassName="text-white hover:bg-indigo-600 bg-indigo-600"
                  pageClassName="bg-white text-indigo-600 hover:bg-indigo-100 transition-colors"
                  pageLinkClassName="h-10 w-10 p-2  duration-150 border border-r-0 border-indigo-600 focus:shadow-outline  flex items-center justify-center"
                  previousLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 rounded-l-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  nextLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-indigo-600 rounded-r-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  breakLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                />
              </div>
            </nav>
          )
          }
        </div>
      </main>
      <Toaster />
      {showModalDelete && (
        <ModalConfirmation
          setShowModal={setShowModalDelete}
          text={`Deseja desativar a condição '${conditionToDelete?.abbr}'?`}
          actionDelete={() => handleDeleteAgent(conditionToDelete?.id)}
          actionCancel={handleCancelDelete}
        />
      )}
    </div>
  );
}


export { ListConditions };