import { useEffect, useState } from "react";

/* Libs */
import { MapContainer, Marker, Popup, TileLayer, Tooltip, useMap, ZoomControl } from "react-leaflet";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";
import useGeolocation from "react-hook-geolocation";
import { NavLink } from "react-router-dom";


/* Components */
import { Sidebar } from "../../components/Sidebar";
import { MapEvent } from "../Patients/CreatePatients";
import { Select } from "../../components/Select";
import { Input } from "../../components/Input";

/* Utils */
import { useAuth } from "../../context/AuthContext";
import { api } from "../../api/axios";
import { arrayColorsMicroareas } from "../../utils/arrayColorsMicroareas";
import { transformJSONToCoordsArray } from "../../utils/transformJSONToCoordsArray";
import mapIcon from "../../utils/mapIcon";

/* Map */
import L from 'leaflet';
import "leaflet-draw";
import "leaflet/dist/leaflet.css";
import "leaflet-draw/dist/leaflet.draw.css";
import { string } from "yup";

interface Coords {
  latitude: number,
  longitude: number,
}

interface MicroareaResponse {
  id: string;
  number: number;
  createdAt: string;
  deletedAt: string | null;
  localization: Coords[];
}


interface PatientResponse {
  id: string;
  name: string;
  cpf: string;
  birthdate: string;
  sex: string;
  microarea: number,
  localization: {
    latitude: number,
    longitude: number,
  },
  conditions: {
    id: string;
    abbr: string;
  }[]
}

/* Dourados - MS*/
const initialPosition = {
  lat: -22.2307329,
  lng: -54.8062403
}

interface MapEventPolygonProps {
  microareas: MicroareaResponse[];
  microareaSelected: {
    coords: L.LatLngExpression[];
    microareaNumber: number;
  };
}
/*Coloca as microáreas no mapa */
function MapEventPolygon({ microareas, microareaSelected }: MapEventPolygonProps) {
  const map = useMap();
  const [polygonDrawer, setPolygonDrawer] = useState<L.Polygon[]>([]);
  const [polygonDrawerSelected, setPolygonDrawerSelected] = useState<L.Polygon>();

  useEffect(() => {
    if (microareaSelected.coords?.length > 0) {
      polygonDrawer.forEach(polygon => {
        map.removeLayer(polygon);
      });

      const coords = microareaSelected.coords;

      const polygon = L.polygon(coords, {
        color: arrayColorsMicroareas[microareaSelected.microareaNumber - 1],
      });

      polygon.bindTooltip(`Microárea ${microareaSelected.microareaNumber}`);

      if (polygonDrawerSelected) {
        map.removeLayer(polygonDrawerSelected);
      }

      map.addLayer(polygon);
      setPolygonDrawerSelected(polygon);
    }
  }, [microareaSelected]);

  useEffect(() => {
    if (microareas?.length > 0 && (microareaSelected.coords?.length <= 0 || microareaSelected.coords?.length === undefined)) {
      if (polygonDrawerSelected) {
        map.removeLayer(polygonDrawerSelected);
        setPolygonDrawerSelected(undefined);
      }

      microareas.forEach(microarea => {
        const coords = transformJSONToCoordsArray(microarea.localization);

        const polygon = L.polygon(coords, {
          color: arrayColorsMicroareas[microarea.number - 1],
        });

        polygon.bindTooltip(`Microárea ${microarea.number}`);

        let newPolygons = [];
        setPolygonDrawer(prev => {
          newPolygons = prev;
          newPolygons.push(polygon);
          return prev;
        })

        map.addLayer(polygon);
      });
    }
  }, [microareas, microareaSelected]);

  return null;
}

function removeAccents(str: string): string {
  str = str.replace(new RegExp('[ÁÀÂÃ]', 'gi'), 'a');
  str = str.replace(new RegExp('[ÉÈÊ]', 'gi'), 'e');
  str = str.replace(new RegExp('[ÍÌÎ]', 'gi'), 'i');
  str = str.replace(new RegExp('[ÓÒÔÕ]', 'gi'), 'o');
  str = str.replace(new RegExp('[ÚÙÛ]', 'gi'), 'u');
  str = str.replace(new RegExp('[Ç]', 'gi'), 'c');
  return str;
}

function Dashboard() {
  const { user } = useAuth();
  const [isOpen, setIsOpen] = useState(false);
  const [position, setPosition] = useState({ latitude: 0, longitude: 0 });
  const [microareas, setMicroareas] = useState<MicroareaResponse[]>([]);
  const [patients, setPatients] = useState<PatientResponse[]>([]);
  const [patientsSelected, setPatientsSelected] = useState<PatientResponse[]>([]);

  const [optionsMicroareaSelect, setOptionsMicroareaSelect] = useState<{ key: string; value: string; }[]>([]);
  const [microareaSelected, setMicroareaSelected] = useState<{ coords: L.LatLngExpression[], microareaNumber: number }>({} as { coords: L.LatLngExpression[], microareaNumber: number });

  const [search, setSearch] = useState('');

  /* Localização do usuário */
  const location = useGeolocation();

  /*Busca todas as microáres */
  useEffect(() => {
    const listMicroareas = async () => {
      try {
        const response = await api.get<MicroareaResponse[]>('/microareas');

        if (response.data) {
          setMicroareas(response.data);

          const options = response.data.map(microarea => {
            return {
              key: microarea.id,
              value: (microarea.number).toString()
            }
          });

          setOptionsMicroareaSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user) {
      listMicroareas();
    }
  }, [user]);

  /* Busca pacientes conforme o usuário */
  useEffect(() => {
    const listAgents = async () => {
      try {
        const response = await api.get<PatientResponse[]>('/patients');

        if (response.data) {
          setPatients(response.data);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listAgents();
  }, [user]);

  /* Atualiza a localização do usuário */
  useEffect(() => {
    if (location?.latitude && location?.longitude && position.latitude === 0) {
      setPosition({
        latitude: location.latitude,
        longitude: location.longitude
      });
    }
  }, [location]);

  /* Atualiza a microárea selecionada */
  const handleSelectSingle = (event: React.FormEvent<HTMLSelectElement>) => {
    if (event.currentTarget.name === "microareaId") {
      const microarea = microareas.find(microarea => microarea.id === event.currentTarget.value);
      if (event.currentTarget.value === "default" || event.currentTarget.value === undefined) {
        setMicroareaSelected({} as { coords: L.LatLngExpression[], microareaNumber: number });

        if (patientsSelected.length > 0 && search.length <= 3) {
          setPatientsSelected([]);
        }
      }

      if (microarea) {
        setMicroareaSelected((prev) => {
          return {
            ...prev,
            coords: transformJSONToCoordsArray(microarea.localization),
            microareaNumber: microarea.number
          }
        });

        const patientsFiltered = patients.filter(patient => patient.microarea === microarea.number);

        setPatientsSelected(patientsFiltered);
      }
    }
  };

  /*Trata de quando há uma busca está sendo realizada */
  const handleChange = (event: React.FormEvent<HTMLInputElement>) => {
    const { value } = event.currentTarget;

    let patientsFiltered = [] as PatientResponse[];
    if (value.length >= 3) {

      if (microareaSelected.coords?.length > 0) {
        patientsFiltered = patientsFiltered.filter(patient => removeAccents(patient.name.toLowerCase()).includes(value.toLowerCase()));
      } else {
        patientsFiltered = patients.filter(patient => removeAccents(patient.name.toLowerCase()).includes(value.toLowerCase()));
      }

      setPatientsSelected(patientsFiltered);
    } else {
      if (microareaSelected.coords?.length <= 0 || microareaSelected.coords === undefined) {
        setPatientsSelected([]);
      } else {
        if (patientsFiltered.length <= 0 && microareaSelected.coords?.length > 0) {
          patientsFiltered = patients.filter(patient => patient.microarea === microareaSelected.microareaNumber);
          setPatientsSelected(patientsFiltered);
        }
      }
    }

    setSearch(value);
  }

  /* Trata de abrir e fechar a Sidebar */
  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }
        <div className="md:h-20 w-full flex flex-col md:flex-row px-2 md:px-0 md:space-x-2 space-y-2 mt-9 mb-1">
          <div className="flex-1 md:self-end md:pl-4 flex justify-center">
            <div className="w-full max-w-xs">
              <Input
                label="Buscar por nome"
                id="district"
                type="text"
                name="district"
                placeholder="Nome do paciente"
                onChange={handleChange}
                value={search || ''}
                inputAddClasses="py-1 px-3"
              />
            </div>
          </div>
          <div className="flex-1 md:self-end md:pr-4 flex flex-col items-center">
            <div className="w-full max-w-xs">
              <Select
                label="Microárea"
                id="microareaId"
                name="microareaId"
                onChange={handleSelectSingle}
                options={optionsMicroareaSelect}
                textDefault="Selecione uma microárea"
                selectAddClasses="py-1.5 px-3"
              />
            </div>
          </div>
        </div>
        <MapContainer
          center={position.latitude ? [position.latitude, position.longitude] : initialPosition}
          zoom={14}
          zoomControl={false}
          className="w-full h-full"
        >
          <MapEvent setPosition={setPosition} />
          <MapEventPolygon microareas={microareas} microareaSelected={microareaSelected} />

          <TileLayer
            attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
            url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
          />
          <ZoomControl position='bottomright' zoomInTitle='Aumentar' zoomOutTitle='Diminuir' />


          {patientsSelected && patientsSelected.map(patient => {
            return (
              <Marker
                key={patient.id}
                position={[patient.localization.latitude, patient.localization.longitude]}
                icon={mapIcon}
              >
                <Popup>
                  <div className="w-full flex items-center justify-center flex-col space-y-2">
                    <span className="font-semibold">{patient.name}</span>
                    <NavLink to={`/patients/update/${patient.id}`} >Ver mais</NavLink>
                  </div>
                </Popup>
                <Tooltip>{patient.name}</Tooltip>
              </Marker>
            );
          })
          }
        </MapContainer>
      </main>
      <Toaster />
    </div>
  );
}

export { Dashboard }