import { AxiosError } from "axios";
import React, { useCallback, useState } from "react";
import { api } from "../../api/axios";
import { Input } from "../../components/Input";
import toast, { Toaster } from 'react-hot-toast';
import { useAuth } from "../../context/AuthContext";
import { useNavigate } from "react-router-dom";
import * as yup from 'yup';

interface UserLogin {
  cpf: string;
  password: string;
}

interface UserLoginErrors {
  cpf: string | null;
  password: string | null;
}


interface UserData {
  id: string;
  role: 'ADMIN' | 'AGENT';
  name: string;
}

interface ResponseData {
  token: string;
  user: UserData;
}

const schema = yup.object().shape({
  cpf: yup.string()
    .required("CPF é obrigatório")
    .max(11, "Preencha o CPF corretamente")
    .min(11, "Preencha o CPF corretamente"),
  password: yup.string()
    .required("Senha é obrigatório"),
});

function Login() {
  const [user, setUser] = useState<UserLogin>({} as UserLogin);
  const [userErrors, setUserErrors] = useState<UserLoginErrors>({
    cpf: null,
    password: null
  } as UserLoginErrors);
  const { signIn } = useAuth();
  const navigate = useNavigate();

  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement>) => {
    setUser({
      ...user,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setUserErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [user]);

  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();

    try {
      const dataValidate = await schema.validate({
        cpf: user.cpf.replace(/(\.|\-)/g, ''),
        password: user.password
      }, {
        abortEarly: false
      });

      const cpfWithoutFormatting = dataValidate.cpf.replace(/(\.|\-)/g, '');

      try {
        const { data } = await api.post<ResponseData>('/users/login', {
          cpf: cpfWithoutFormatting,
          password: user.password,
        });

        signIn(data.user, data.token);

        navigate('/dashboard');
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-right'
            })
          }
        }
      }
    } catch (err) {
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setUserErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  return (
    <div className="min-h-screen bg-gray-100 flex flex-col justify-center items-center text-gray-800 antialiased py-6">
      <div className="relative py-3 sm:w-96 sm:max-w-xl sm:mx-auto">
        <h2 className="text-2xl font-medium text-center">Faça seu login</h2>
        <div className="mt-4 bg-white shadow-md rounded-lg">
          <div className="h-2 bg-cyan-400 rounded-t-md"></div>
          <form onSubmit={handleSubmit}>
            <div className="px-8 py-6 flex flex-col space-y-6">
              <div className="mb-2">
                <Input
                  label="CPF"
                  id="cpf"
                  type="text"
                  name="cpf"
                  mask="CPF"
                  onChange={handleChange}
                  placeholder="999.999.999-99"
                  error={userErrors.cpf ? userErrors.cpf : ''}
                />
              </div>
              <div className="mb-4">
                <Input
                  label="Senha"
                  id="password"
                  type="password"
                  name="password"
                  onChange={handleChange}
                  placeholder="Digite sua senha"
                  error={userErrors.password ? userErrors.password : ''}
                />
              </div>

              <div className="flex justify-between items-baseline">
                <button type="submit" className="w-full inline-flex items-center justify-center mt-4 bg-cyan-500 text-white px-3 py-2  rounded-md hover:bg-cyan-600 focus:outline-none focus:border-cyan-700 focus:ring focus:ring-cyan-200 disabled:opacity-25 transition">Entrar</button>
              </div>
            </div>
          </form>
        </div>
      </div>
      <Toaster />
    </div>
  );
}

export { Login };