import { FormEvent, useCallback, useEffect, useRef, useState } from "react";
import { useNavigate } from "react-router-dom";
import { Sidebar } from "../../../components/Sidebar";
import * as yup from 'yup';

import useGeolocation from "react-hook-geolocation";
import toast, { Toaster } from "react-hot-toast";
import { WarningCircle } from "phosphor-react";
import { Input } from "../../../components/Input";
import { api } from "../../../api/axios";
import { AxiosError } from "axios";
import { useAuth } from "../../../context/AuthContext";
import { transformJSONToCoordsArray } from "../../../utils/transformJSONToCoordsArray";

import L from 'leaflet';
import "leaflet-draw";
import "leaflet/dist/leaflet.css";
import "leaflet-draw/dist/leaflet.draw.css";

interface MicroareaCoords {
  id: number,
  latlngs: L.LatLngLiteral[]
}

interface MicroareaForm {
  number: number;
  localization: L.LatLngLiteral[];
}

interface MicroareaFormError {
  number: string | null;
  localization: string | null;
}

interface Coords {
  latitude: number,
  longitude: number,
}

interface MicroareaResponse {
  id: string;
  number: number;
  createdAt: string;
  deletedAt: string | null;
  localization: Coords[];
}

/* Dourados - MS*/
const initialPosition = {
  lat: -22.2307329,
  lng: -54.8062403
}

const schema = yup.object().shape({
  number: yup
    .number()
    .typeError('Precisa ser um número.')
    .required("Número de microárea é obrigatório.")
    .min(1, "O Número mínimo é 1.")
    .lessThan(12, "Número de microárea não pode ser maior que 12."),
});


function CreateMicroareas() {
  const { user } = useAuth();
  const navigate = useNavigate();
  const [isOpen, setIsOpen] = useState(false);
  /* Localização do usuário */
  const location = useGeolocation();

  const [polygon, setPolygon] = useState<MicroareaCoords>({} as MicroareaCoords);
  const [microareaCoords, setMicroareaCoords] = useState<L.LatLngLiteral[]>([]);

  const myStateRef = useRef(polygon);
  const setMyState = (data: MicroareaCoords) => {
    myStateRef.current = data;
    setPolygon(data);
  };

  const [microarea, setMicroarea] = useState<MicroareaForm>({} as MicroareaForm);
  const [microareaErrors, setMicroareaErrors] = useState<MicroareaFormError>({
    number: null,
    localization: null,
  } as MicroareaFormError);

  const [microareas, setMicroareas] = useState<MicroareaResponse[]>([]);


  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  useEffect(() => {
    if (polygon.id) {
      const microarea = polygon.latlngs.map(latlng => {
        return {
          lat: latlng.lat,
          lng: latlng.lng,
        }
      })
      setMicroareaCoords(microarea);
      setMicroareaErrors((prevState) => {
        return {
          ...prevState,
          "localization": null
        }
      });
    }

    return
  }, [polygon]);

  /* Buscar todas as microáreas já criadas */
  useEffect(() => {
    const listMicroareas = async () => {
      try {
        const response = await api.get<MicroareaResponse[]>('/microareas');

        if (response.data) {
          setMicroareas(response.data);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user) {
      listMicroareas();
    }
  }, [user]);

  useEffect(() => {
    let position: L.LatLngExpression = initialPosition;
    if (location) {
      position.lat = location.latitude;
      position.lng = location.longitude;

    }
    const map = L.map('map').setView(position, 13);

    L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
      maxZoom: 19,
      attribution: '© OpenStreetMap'
    }).addTo(map);

    const drawnItems = new L.FeatureGroup();
    map.addLayer(drawnItems);

    const drawControl = new L.Control.Draw({
      edit: {
        featureGroup: drawnItems,
      },
      position: "topright",
      draw: {
        circle: false,
        circlemarker: false,
        marker: false,
        polyline: false,
        rectangle: false
      }
    });
    map.addControl(drawControl);

    /* Adicionando as microáreas já existentes */
    if (microareas && microareas.length > 0) {
      microareas.forEach(microarea => {
        const coords = transformJSONToCoordsArray(microarea.localization);
        const polygon = L.polygon(coords, {
          color: "red"
        });

        polygon.bindTooltip(`Microárea ${microarea.number}`);

        map.addLayer(polygon);
      });
    }

    /* Eventos de desenho */
    map.addEventListener(L.Draw.Event.CREATED, (event) => {
      const isCreateLayer = handleCreateDraw(event);

      if (!isCreateLayer) {
        return;
      }

      drawnItems.addLayer(event.layer);
    });

    map.addEventListener(L.Draw.Event.EDITED, (event) => {
      handleEditDraw(event);
    });

    map.addEventListener(L.Draw.Event.DELETED, (event) => {
      handleDeleteDraw(event);
    });

    map.addEventListener(L.Draw.Event.DRAWVERTEX, (event) => {
      handleDrawStart(event);
    });

    return () => {
      map.removeEventListener(L.Draw.Event.CREATED);
      map.removeEventListener(L.Draw.Event.EDITED);
      map.removeEventListener(L.Draw.Event.DELETED);
      map.removeEventListener(L.Draw.Event.DRAWVERTEX);
      map.off();
      map.remove();
    }

  }, [microareas, location]);

  /*  Adicionar em um estado o draw */
  const handleCreateDraw = (event: L.LeafletEvent) => {
    const { layer, layerType } = event as any;
    if (layerType === "polygon") {
      const coords = layer.getLatLngs();

      if (myStateRef.current.latlngs?.length > 0) {
        return false;
      }
      setMyState({
        id: 202,
        latlngs: [...coords[0]],
      });

      return true;
    }
  }

  /* Verificar se há um draw já existente e alertar */
  const handleDrawStart = (event: L.LeafletEvent) => {

    if (myStateRef.current.latlngs?.length > 0) {
      toast("Microárea já criada! Edite ou exclua a existente!", {
        duration: 4000,
        position: 'top-center',
        icon: <WarningCircle color="#f97316" size={40} weight="fill" />
      });
    }

    return;
  }

  /* Trata quando há uma edição na área desenhada */
  const handleEditDraw = (event: L.LeafletEvent) => {
    const { layers } = event as any;
    const { _layers } = layers;


    Object.values(_layers).map((coord: any) => {

      const newPolygon = coord.getLatLngs();

      if (newPolygon.length > 0) {
        setMyState({
          ...myStateRef.current,
          latlngs: newPolygon[0],
        });
      }
    });
  }

  /* Trata quando há uma deleção de uma área desenhada */
  const handleDeleteDraw = (event: L.LeafletEvent) => {
    const { layers } = event as any;
    const { _layers } = layers;

    Object.values(_layers).map((coord: any) => {


      setMyState({} as MicroareaCoords);

    });

    setMicroareaCoords([]);
  }

  /* Trata quando há alterações nos campos do formulário */
  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement> | React.FormEvent<HTMLTextAreaElement>) => {
    setMicroarea({
      ...microarea,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setMicroareaErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [microarea]);


  /*Fazer o submit do form */
  const handleSubmit = async (event: FormEvent) => {
    event.preventDefault();

    try {
      /* validando os dados */
      const dataValidate = await schema.validate({
        number: microarea.number,
      }, {
        abortEarly: false
      });


      if (microareaCoords.length <= 0) {
        /* Não há microáreas selecionadas */
        setMicroareaErrors((prevState) => {
          return {
            ...prevState,
            "localization": "Adicione uma microárea."
          }
        });
        return;
      }

      /* Cadastrar microárea */
      try {
        const response = await api.post('/microareas', {
          ...dataValidate,
          localization: microareaCoords,
        });

        if (response.status === 202) {
          navigate('/microareas/list');
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            });
          }
        }
      }

    } catch (err) {
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setMicroareaErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  const handleCancel = () => {
    navigate('/dashboard');
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>

      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }

        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Criar microáreas</h2>
        </div>

        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">

          <div className="-mx-3 mb-2 px-3">
            <span>Crie uma microárea adicionado um poligono</span>
          </div>
          <div className="w-full h-[350px] sm:h-[450px] lg:h-[550px] -mx-3 mb-6 px-3">
            <div id="map" className="w-full h-full"></div>
            {microareaErrors.localization && (
              <span className="mt-1 text-red-500 text-xs italic">{microareaErrors.localization}</span>
            )}
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Número da microárea"
                id="number"
                type="number"
                name="number"
                placeholder="Coloque o número da microárea"
                error={microareaErrors.number ? microareaErrors.number : ''}
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { CreateMicroareas }