import { useCallback, useEffect, useState } from "react";
import { MapContainer, Marker, TileLayer, useMapEvent, ZoomControl, useMap } from "react-leaflet";
import { useNavigate } from "react-router-dom";
import { Input } from "../../../components/Input";
import { Select } from "../../../components/Select";
import { Sidebar } from "../../../components/Sidebar";
import { Textarea } from "../../../components/Textarea";
import useGeolocation from "react-hook-geolocation";
import L, { LeafletMouseEvent } from 'leaflet';
import "leaflet-draw";
import "leaflet/dist/leaflet.css";
import "leaflet-draw/dist/leaflet.draw.css";
import MapIcon from '../../../utils/mapIcon';
import { AxiosError } from "axios";
import { api } from "../../../api/axios";
import toast, { Toaster } from "react-hot-toast";
import { useAuth } from "../../../context/AuthContext";
import { transformJSONToCoordsArray } from "../../../utils/transformJSONToCoordsArray";
import MultiSelect, { ActionMeta, MultiValue } from 'react-select';
import { multiSelectStyles } from "../../../utils/multiSelectStyles";
import * as yup from 'yup';
import { format } from "date-fns";


export interface PatientForm {
  name: string;
  cpf: string;
  sex: string;
  birthdate: string;
  observation: string;
  publicPlace: string;
  number: number;
  complement: string;
  district: string;
  microareaId: string;
  conditions: string[];
  position: {
    latitude: number,
    longitude: number,
  }
}

export interface PatientFormErrors {
  name: string | null;
  cpf: string | null;
  sex: string | null;
  birthdate: string | null;
  publicPlace: string | null;
  number: string | null;
  district: string | null;
  microareaId: string | null;
}

interface Coords {
  latitude: number,
  longitude: number,
}

interface MicroareaResponse {
  id: string;
  number: number;
  createdAt: string;
  deletedAt: string | null;
  localization: Coords[];
}

interface ConditionResponse {
  id: string;
  abbr: string;
  description: string;
  deletedAt: string | null;
}

const optionsSexSelect = [
  { key: "FEMININO", value: "Feminino" },
  { key: "MASCULINO", value: "Masculino" },
];

/* Dourados - MS*/
const initialPosition = {
  lat: -22.2307329,
  lng: -54.8062403
}

type Position = {
  latitude: number,
  longitude: number
}

type MapEventProps = {
  setPosition: (position: Position) => void;
}

export function MapEvent({ setPosition }: MapEventProps) {
  const map = useMapEvent('click', (event: LeafletMouseEvent) => {
    const { latlng } = event;
    setPosition({
      latitude: latlng.lat,
      longitude: latlng.lng
    });
  });
  return null;
}

export function MapEventFlyTo(position: Position) {
  const map = useMap();
  const [moved, setMoved] = useState(false);

  useEffect(() => {
    if (!moved && position.latitude !== 0 && position.longitude !== 0) {
      map.flyTo([position.latitude, position.longitude], 15);
      setMoved(true);
    }
  }, [position, map]);

  return null;
}

interface MapEventPolygonProps {
  coords: L.LatLngExpression[];
}
export function MapEventPolygon({ coords }: MapEventPolygonProps) {
  const map = useMap();
  const [polygonDrawer, setPolygonDrawer] = useState<L.Polygon>();

  useEffect(() => {
    if (coords?.length > 0) {
      const polygon = L.polygon(coords, {
        color: "red"
      });

      if (polygonDrawer) {
        map.removeLayer(polygonDrawer);
      }

      setPolygonDrawer(polygon);
      map.addLayer(polygon);
    }
  }, [coords])



  return null;
}

const schema = yup.object().shape({
  name: yup
    .string()
    .required("Nome é obrigatório")
    .max(255, "Nome deve ter no máximo 255 caracteres")
    .min(2, "Nome deve ter no mínimo 2 caracteres"),
  cpf: yup
    .string()
    .required("CPF é obrigatório")
    .max(11, "Preencha o CPF corretamente")
    .min(11, "Preencha o CPF corretamente"),
  birthdate: yup
    .date()
    .required("Data de nascimento é obrigatório")
    .max(new Date(), "Data de nascimento inválida"),
  publicPlace: yup
    .string()
    .required("Logradouro é obrigatório")
    .max(255, "Logradouro deve ter no máximo 255 caracteres")
    .min(2, "Logradouro deve ter no mínimo 2 caracteres"),
  number: yup
    .number()
    .typeError("Precisa ser um número")
    .required("Número é obrigatório")
    .min(1, "O Número mínimo é 1")
    .lessThan(99999, "Número da casa não pode ser maior que 99999"),
  district: yup
    .string()
    .required("Bairro é obrigatório")
    .max(255, "Bairro deve ter no máximo 255 caracteres")
    .min(2, "Bairro deve ter no mínimo 2 caracteres"),
});

export { schema as patientSchema };

function CreatePatients() {
  const { user } = useAuth();
  const navigate = useNavigate();
  const [isOpen, setIsOpen] = useState(false);
  const [position, setPosition] = useState({ latitude: 0, longitude: 0 });
  const [patient, setPatient] = useState<PatientForm>({} as PatientForm);

  const [patientErrors, setPatientErrors] = useState<PatientFormErrors>({
    cpf: null,
    name: null,
    sex: null,
    birthdate: null,
    publicPlace: null,
    number: null,
    district: null,
    microareaId: null
  } as PatientFormErrors);
  const [optionsMicroareaSelect, setOptionsMicroareaSelect] = useState<{ key: string; value: string; }[]>([]);

  const [microareas, setMicroareas] = useState<MicroareaResponse[]>([]);
  const [microareaSelected, setMicroareaSelected] = useState<{ coords: L.LatLngExpression[], microareaNumber: number }>({} as { coords: L.LatLngExpression[], microareaNumber: number });
  const [optionsConditionsSelect, setOptionsConditionsSelect] = useState<{ value: string; label: string; }[]>([]);


  /* Localização do usuário */
  const location = useGeolocation();

  /* Buscar todas as microáreas já criadas */
  useEffect(() => {
    const listMicroareas = async () => {
      try {
        const response = await api.get<MicroareaResponse[]>('/microareas');

        if (response.data) {
          setMicroareas(response.data);

          const options = response.data.map(microarea => {
            return {
              key: microarea.id,
              value: (microarea.number).toString()
            }
          });

          setOptionsMicroareaSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user) {
      listMicroareas();
    }
  }, [user]);

  /* Busca todas as condições de saúde existentes */
  useEffect(() => {
    const listConditons = async () => {
      try {
        const response = await api.get<ConditionResponse[]>('/conditions');

        if (response.data) {
          let options = [] as { value: string; label: string; }[];
          response.data.forEach((condition) => {
            if (condition.deletedAt === null) { // Verifica se a condição está ativa
              options.push({
                value: condition.id,
                label: condition.abbr
              });
            }
          })
          setOptionsConditionsSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listConditons();
  }, [user]);

  useEffect(() => {
    if (location?.latitude && location?.longitude && position.latitude === 0) {
      setPosition({
        latitude: location.latitude,
        longitude: location.longitude
      });
    }
  }, [location]);

  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement> | React.FormEvent<HTMLTextAreaElement>) => {
    setPatient({
      ...patient,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [patient]);

  const handleSelectSingle = (event: React.FormEvent<HTMLSelectElement>) => {
    setPatient({
      ...patient,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (event.currentTarget.name === "microareaId") {
      const microarea = microareas.find(microarea => microarea.id === event.currentTarget.value);
      if (microarea) {
        setMicroareaSelected((prev) => {
          return {
            ...prev,
            coords: transformJSONToCoordsArray(microarea.localization),
            microareaNumber: microarea.number
          }
        });
      }
    }

    if (!!event.currentTarget.name && event.currentTarget.value !== 'default') {
      let key = event.currentTarget.name;
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  };

  const handleSelectMulti = (newValue: MultiValue<{ value: string, label: string }>, actionMeta: ActionMeta<any>) => {
    const conditionsCurrent = newValue.map(condition => condition.value);
    setPatient({
      ...patient,
      conditions: conditionsCurrent,
    });

    if (!!newValue && newValue.length > 0) {
      let key = "conditions";
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }

  /* Cancelar Cadastro de paciente e retornar para a página inicial */
  const handleCancel = () => {
    navigate('/dashboard');
  }

  /* Cadastrar o novo paciente, chamando a rota do banco de dados */
  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();

    let isValid = true;

    try {
      /* Validando os Selects */
      if (!patient.sex || patient.sex === 'default') {
        setPatientErrors((prevState) => {
          return {
            ...prevState,
            sex: "Este campo é obrigatório"
          }
        });
        isValid = false;
      }

      if (!patient.microareaId || patient.microareaId === 'default') {
        setPatientErrors((prevState) => {
          return {
            ...prevState,
            microareaId: "Este campo é obrigatório"
          }
        });
        isValid = false;
      }

      /* Validando os demais campos usando o Yup */
      const dataValidateByYup = await schema.validate({
        name: patient.name,
        cpf: patient.cpf.replace(/(\.|\-)/g, ''),
        birthdate: patient.birthdate,
        publicPlace: patient.publicPlace,
        number: patient.number,
        district: patient.district,
      }, {
        abortEarly: false
      });


      if (!isValid) {
        return;
      }

      const data = {
        name: dataValidateByYup.name,
        cpf: dataValidateByYup.cpf.replace(/(\.|\-)/g, ''),
        sex: patient.sex,
        birthdate: format(dataValidateByYup.birthdate, "yyyy-MM-dd"),
        observation: patient.observation || "",
        conditions: patient.conditions,
        address: {
          publicPlace: dataValidateByYup.publicPlace,
          number: dataValidateByYup.number,
          district: dataValidateByYup.district,
          complement: patient.complement || "",
        },
        microareaId: patient.microareaId,
        localization: {
          lat: position.latitude,
          lng: position.longitude
        },
      };

      try {
        const response = await api.post('/patients', data);

        if (response.status === 202) {
          navigate('/patients/list');
        }

      } catch (error) {
        /* Erro emitido pelo servidor */
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }

    } catch (err) {

      /* Erro na validação dos campos pelo yup */
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setPatientErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row md:fixed">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex mm:ml-[250px]` : `flex-1 flex relative flex-col`}>
        {isOpen &&
          (<div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>)
        }
        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Cadastro de Paciente</h2>
        </div>


        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Informações Gerais</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Nome Completo"
                id="name"
                type="text"
                name="name"
                placeholder="Nome completo"
                error={patientErrors.name ? patientErrors.name : ''}
                onChange={handleChange}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="CPF"
                id="cpf"
                type="text"
                name="cpf"
                mask="CPF"
                placeholder="999.999.999-99"
                onChange={handleChange}
                error={patientErrors.cpf ? patientErrors.cpf : ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Select
                label="Sexo"
                id="sex"
                name="sex"
                onChange={handleSelectSingle}
                options={optionsSexSelect}
                textDefault="Selecione um sexo"
                error={patientErrors.sex ? patientErrors.sex : ''}
              />
            </div>
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Input
                label="Data de Nascimento"
                id="birthdate"
                type="date"
                name="birthdate"
                onChange={handleChange}
                error={patientErrors.birthdate ? patientErrors.birthdate : ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Textarea
                label="Observação"
                id="observation"
                name="observation"
                placeholder="Digite uma observação sobre o paciente"
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <label className="block mb-1" htmlFor="conditions">Condição de Saúde</label>
              <MultiSelect
                className="w-full"
                options={optionsConditionsSelect}
                isMulti
                id="conditions"
                name="conditions"
                styles={multiSelectStyles}
                placeholder="Selecione as condições de saúde do paciente"
                noOptionsMessage={() => "Não há condições de saúde"}
                onChange={handleSelectMulti}
              />
            </div>
          </div>

          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Endereço</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Input
                label="Logradouro"
                id="publicPlace"
                type="text"
                name="publicPlace"
                placeholder="Digite o nome da rua"
                error={patientErrors.publicPlace ? patientErrors.publicPlace : ''}
                onChange={handleChange}
              />
            </div>
            <div className="w-full md:w-auto px-3">
              <Input
                label="Número"
                id="number"
                type="number"
                name="number"
                placeholder="N°"
                error={patientErrors.number ? patientErrors.number : ''}
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Bairro"
                id="district"
                type="text"
                name="district"
                placeholder="Nome do bairro"
                error={patientErrors.district ? patientErrors.district : ''}
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Textarea
                label="Complemento"
                id="complement"
                name="complement"
                resize
                placeholder="Digite o complemento"
                onChange={handleChange}
              />
            </div>
          </div>

          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Geolocalização</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Select
                label="Microárea"
                id="microareaId"
                name="microareaId"
                onChange={handleSelectSingle}
                options={optionsMicroareaSelect}
                textDefault="Selecione uma microárea"
                error={patientErrors.microareaId ? patientErrors.microareaId : ''}
              />
            </div>
          </div>

          {/* Mapa para geolocalização */}
          <div className=" -mx-3 mb-6 px-3">
            <span className="block mb-1">Ajuste o marcador para ser mais precisa a localização</span>
            <div className="w-full h-[350px] sm:h-[450px] lg:h-[550px]">
              <MapContainer
                center={position.latitude ? [position.latitude, position.longitude] : initialPosition}
                zoom={14}
                zoomControl={false}
                className="w-full h-full"
              >
                <MapEvent setPosition={setPosition} />
                <MapEventFlyTo latitude={position.latitude} longitude={position.longitude} />
                <MapEventPolygon coords={microareaSelected.coords} />
                <TileLayer
                  attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                  url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
                />
                <ZoomControl position='bottomright' zoomInTitle='Aumentar' zoomOutTitle='Diminuir' />

                <Marker
                  interactive={false}
                  icon={MapIcon}
                  position={[position.latitude, position.longitude]}
                />
              </MapContainer>
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { CreatePatients };