import { useEffect, useState } from "react";
import { useNavigate } from "react-router-dom";

/* Libs */
import { CaretLeft, CaretRight, DotsThree } from "phosphor-react";
import ReactPaginate from "react-paginate";
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";

/* Components */
import { Sidebar } from "../../../components/Sidebar";

/* Utils */
import { CPFMaskToView } from "../../../utils/masks";
import { api } from "../../../api/axios";
import { useAuth } from "../../../context/AuthContext";
import { calculateAge } from "../../../utils/calculateAge";


/* Interfaces */
interface PatientResponse {
  id: string;
  name: string;
  cpf: string;
  birthdate: string;
  sex: string;
  microarea: number,
  localization: {
    latitude: number,
    longitude: number,
  },
  conditions: {
    id: string;
    abbr: string;
  }[]
}


function ListPatients() {
  const { user } = useAuth();
  const navigate = useNavigate();

  const [isOpen, setIsOpen] = useState(false);

  const [patients, setPatients] = useState<PatientResponse[]>([]);

  /*Para a paginação*/
  const [pageCount, setPageCount] = useState(0);
  const [itemOffset, setItemOffset] = useState(0);
  const [currentItems, setCurrentItems] = useState<PatientResponse[] | null>(null);
  const itemsPerPage = 10;

  /* Troca de página */
  useEffect(() => {
    if (patients) {
      const endOffset = itemOffset + itemsPerPage;

      setCurrentItems(patients.slice(itemOffset, endOffset));
      setPageCount(Math.ceil(patients.length / itemsPerPage));
    }
  }, [itemOffset, itemsPerPage, patients]);

  /* Busca pacientes conforme o usuário */
  useEffect(() => {
    const listAgents = async () => {
      try {
        const response = await api.get<PatientResponse[]>('/patients');

        if (response.data) {
          setPatients(response.data);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listAgents();
  }, [user]);

  /* Abre e fecha Sidebar */
  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  /* Faz a troca de página */
  const handlePageClick = (event: { selected: number }) => {
    const newOffset = event.selected * itemsPerPage % patients.length;
    setItemOffset(newOffset);
  };


  /* Faz o redirect para a página de atualização de paciente */
  const handleUpdatePatient = (patientId: string) => {
    navigate(`/patients/update/${patientId}`);
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>

      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen &&
          (<div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>)
        }

        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Lista de Pacientes</h2>
        </div>

        <div className="overflow-x-auto relative w-full px-5 lg:px-8 bg-gray-50 py-6 lg:max-w-[90%] mx-auto mb-5">
          <table className="w-full text-sm text-left text-gray-500 dark:text-gray-400">
            <thead className="sticky top-0 text-xs text-gray-700 uppercase bg-gray-50 dark:bg-gray-700 dark:text-gray-400">
              <tr>
                <th scope="col" className="py-3 px-6">
                  CPF
                </th>
                <th scope="col" className="py-3 px-6">
                  Nome
                </th>
                <th scope="col" className="py-3 px-6">
                  Microárea
                </th>
                <th scope="col" className="py-3 px-6">
                  Idade
                </th>
                <th scope="col" className="py-3 px-6">
                  Ações
                </th>
              </tr>
            </thead>
            <tbody className="overflow-x-auto ">
              {currentItems?.map((patient) => {
                return (
                  <tr key={patient.id} className="bg-white border-b dark:bg-gray-800 dark:border-gray-700 hover:bg-gray-50 dark:hover:bg-gray-600">
                    <td scope="row" className="py-4 px-6 font-medium text-gray-900 whitespace-nowrap dark:text-white">
                      {CPFMaskToView(patient.cpf)}
                    </td>
                    <td className="py-4 px-6">
                      {patient.name}
                    </td>
                    <td className="py-4 px-6">
                      {patient.microarea}
                    </td>
                    <td className="py-4 px-6">
                      {calculateAge(patient.birthdate)}
                    </td>
                    <td className="py-4 px-6">
                      <div className="w-full px-3 flex space-x-2">
                        <button className="btn_success" onClick={() => handleUpdatePatient(patient.id)}>Alterar</button>
                      </div>
                    </td>
                  </tr>
                );
              })}
            </tbody>
          </table>

          {patients.length > itemsPerPage && (
            <nav className="flex justify-between items-center pt-4" aria-label="Table navigation">
              <span className="text-sm font-normal text-gray-500 dark:text-gray-400">Listando <span className="font-semibold text-gray-900 dark:text-white">{itemOffset === 0 ? 1 : itemOffset + 1} - {(itemsPerPage + itemOffset) > patients.length ? patients.length : (itemsPerPage + itemOffset)}</span> de <span className="font-semibold text-gray-900 dark:text-white">{patients.length}</span></span>
              <div className="inline-flex items-center -space-x-px">
                <ReactPaginate
                  pageCount={pageCount}
                  onPageChange={handlePageClick}
                  pageRangeDisplayed={2}
                  marginPagesDisplayed={1}
                  nextLabel={<CaretRight size={20} />}
                  previousLabel={<CaretLeft size={20} />}
                  breakLabel={<DotsThree size={32} />}
                  containerClassName="inline-flex"
                  activeClassName="bg-indigo-600"
                  activeLinkClassName="text-white hover:bg-indigo-600 bg-indigo-600"
                  pageClassName="bg-white text-indigo-600 hover:bg-indigo-100 transition-colors"
                  pageLinkClassName="h-10 w-10 p-2  duration-150 border border-r-0 border-indigo-600 focus:shadow-outline  flex items-center justify-center"
                  previousLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 rounded-l-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  nextLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-indigo-600 rounded-r-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  breakLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                />
              </div>
            </nav>
          )
          }
        </div>
      </main>
      <Toaster />
    </div>
  );
}


export { ListPatients };