import { FormEvent, useEffect, useState } from "react";

/* Libs */
import { AxiosError } from "axios";
import toast, { Toaster } from "react-hot-toast";
import MultiSelect, { ActionMeta, MultiValue } from 'react-select';
import { CaretLeft, CaretRight, DotsThree, FilePdf, MinusCircle, PlusCircle } from "phosphor-react";
import ReactPaginate from "react-paginate";



/* Components */
import { Select } from "../../../components/Select";
import { Sidebar } from "../../../components/Sidebar";
import { TitlePage } from "../../../components/TitlePage";
import { useAuth } from "../../../context/AuthContext";

/* Utils */
import { api } from "../../../api/axios";
import { multiSelectStyles } from "../../../utils/multiSelectStyles";
import { CPFMaskToView } from "../../../utils/masks";
import { calculateAge } from "../../../utils/calculateAge";
import { useNavigate } from "react-router-dom";
import { patientReportPDF } from "../../../reports/Patients/patients";

/* Interfaces */
interface AgentResponse {
  id: string;
  name: string;
  cpf: string;
  role: string;
  deletedAt: string | null;
}

interface Coords {
  latitude: number,
  longitude: number,
}

interface MicroareaResponse {
  id: string;
  number: number;
  createdAt: string;
  deletedAt: string | null;
  localization: Coords[];
}

interface ConditionResponse {
  id: string;
  abbr: string;
  description: string;
  deletedAt: string | null;
}

interface PatientResponse {
  id: string;
  name: string;
  cpf: string;
  birthdate: string;
  sex: string;
  microarea: number,
  localization: {
    latitude: number,
    longitude: number,
  },
  conditions: {
    id: string;
    abbr: string;
  }[]
}

interface SearchData {
  user: string;
  microarea: string;
  under_five_years_old: string;
  conditions: string[];
}

function SearchPatients() {
  const { user } = useAuth();
  const navigate = useNavigate();

  const [isOpen, setIsOpen] = useState(false);
  const [search, setSearch] = useState<SearchData>({
    user: "",
    microarea: "",
    conditions: [],
    under_five_years_old: "false",
  });

  const [optionsUserSelect, setOptionsUserSelect] = useState<{ key: string; value: string; }[]>([]);
  const [optionsMicroareaSelect, setOptionsMicroareaSelect] = useState<{ key: string; value: string; }[]>([]);
  const [optionsConditionsSelect, setOptionsConditionsSelect] = useState<{ value: string; label: string; }[]>([]);

  const [isAdvancedSearch, setIsAdvancedSearch] = useState(false);

  const [patients, setPatients] = useState<PatientResponse[]>([]);

  /*Para a paginação*/
  const [pageCount, setPageCount] = useState(0);
  const [itemOffset, setItemOffset] = useState(0);
  const [currentItems, setCurrentItems] = useState<PatientResponse[] | null>(null);
  const itemsPerPage = 5;

  /* Troca de página */
  useEffect(() => {
    if (patients) {
      const endOffset = itemOffset + itemsPerPage;

      setCurrentItems(patients.slice(itemOffset, endOffset));
      setPageCount(Math.ceil(patients.length / itemsPerPage));
    }
  }, [itemOffset, itemsPerPage, patients]);

  /* Trás todos os agentes */
  useEffect(() => {
    const listAgents = async () => {
      try {
        const response = await api.get<AgentResponse[]>('/users/agents');

        if (response.data) {
          const options = response.data.map(agent => {
            return {
              key: agent.id,
              value: agent.name,
            }
          })

          setOptionsUserSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listAgents();
  }, [user]);

  /* Buscar todas as microáreas já criadas */
  useEffect(() => {
    const listMicroareas = async () => {
      try {
        const response = await api.get<MicroareaResponse[]>('/microareas');

        if (response.data) {

          const options = response.data.map(microarea => {
            return {
              key: microarea.number.toString(),
              value: (microarea.number).toString()
            }
          });

          setOptionsMicroareaSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user) {
      listMicroareas();
    }
  }, [user]);

  /* Busca todas as condições de saúde existentes */
  useEffect(() => {
    const listConditons = async () => {
      try {
        const response = await api.get<ConditionResponse[]>('/conditions');

        if (response.data) {
          const options = response.data.map(condition => {
            return {
              value: condition.id,
              label: condition.abbr,
            }
          })

          setOptionsConditionsSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listConditons();
  }, [user]);



  /* Atualiza os campos de busca - Select */
  const handleSelectSingle = (event: React.FormEvent<HTMLSelectElement>) => {
    setSearch({
      ...search,
      [event.currentTarget.name]: event.currentTarget.value,
    });
  };

  /* Atualiza o campo de busca para condições de saúde - MultiSelect */
  const handleSelectMulti = (newValue: MultiValue<{ value: string, label: string }>, actionMeta: ActionMeta<any>) => {
    const conditionsCurrent = newValue.map(condition => condition.value);
    setSearch({
      ...search,
      conditions: conditionsCurrent,
    });
  }

  /* Faz o redirect para a página de atualização de paciente */
  const handleUpdatePatient = (patientId: string) => {
    navigate(`/patients/update/${patientId}`);
  }

  /* Faz a troca de página */
  const handlePageClick = (event: { selected: number }) => {
    const newOffset = event.selected * itemsPerPage % patients.length;
    setItemOffset(newOffset);
  };

  /* Limpar os campos de busca */
  const handleClear = () => {
    setSearch({
      user: "",
      microarea: "",
      conditions: [],
      under_five_years_old: "false",
    });
    setPatients([]);
  }

  /* Chama a funcção que gera PDF*/
  const handleGeneratePDF = () => {
    if (patients) {
      patientReportPDF(patients);
    }
  }

  /* Abre e Fecha Sidebar no responsivo */
  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  /* Faz busca e retorna os resultados */
  const handleSubmit = async (event: FormEvent) => {
    event.preventDefault();


    const params = {
      conditions: search.conditions.length > 0 ? search.conditions.toString() : "",
      user: search.user === "default" ? "" : search.user,
      microarea: search.microarea === "default" ? "" : search.microarea,
      under_five_years_old: search.under_five_years_old,
    }

    try {
      const response = await api.get<PatientResponse[]>('patients/search', {
        params,
      });

      if (response.data) {
        setPatients(response.data);
      }

    } catch (error) {
      /* Erro emitido pelo servidor */
      if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
        if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
          toast.error(error.response.data.message, {
            duration: 4000,
            position: 'top-center'
          })
        }
      }
    }
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row">
        <Sidebar isOpen={isOpen} setIsOpen={setIsOpen} handleOpenSidebar={handleOpenSidebar} />
      </div>

      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex` : `flex-1 flex relative flex-col`}>
        {isOpen &&
          (<div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>)
        }

        <TitlePage title="Busca de Pacientes" />

        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h3 className="text-lg md:text-xl">Personalize sua busca</h3>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Select
                label="Agentes"
                id="user"
                name="user"
                onChange={handleSelectSingle}
                options={optionsUserSelect}
                textDefault="Selecione um agente"
                value={search.user ? search.user : "default"}
              />
            </div>
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Select
                label="Microárea"
                id="microarea"
                name="microarea"
                onChange={handleSelectSingle}
                options={optionsMicroareaSelect}
                textDefault="Selecione uma microárea"
                value={search.microarea ? search.microarea : "default"}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full px-3 md:w-auto md:flex-1">
              <label className="block mb-1" htmlFor="conditions">Condição de Saúde</label>
              <MultiSelect
                className="w-full"
                options={optionsConditionsSelect}
                isMulti
                id="conditions"
                name="conditions"
                styles={multiSelectStyles}
                placeholder="Selecione as condições de saúde"
                noOptionsMessage={() => "Não há condições de saúde"}
                onChange={handleSelectMulti}
                value={search.conditions.length <= 0 ? [] : optionsConditionsSelect.filter(option => search.conditions.includes(option.value))}
              />
            </div>
            {
              isAdvancedSearch && (
                <div className="w-full md:w-auto md:flex-1 px-3 flex items-center">
                  <div className="items-center flex">
                    <input
                      id="active"
                      name="active"
                      type="checkbox"
                      onChange={(e) => setSearch({
                        ...search,
                        under_five_years_old: e.target.checked ? "true" : "false"
                      })}
                      className="h-4 w-4  bg-cyan-500 border border-gray-300 focus:border-cyan-300 focus:outline-none focus:ring focus:ring-cyan-200 focus:ring-opacity-50 rounded-md shadow-sm disabled:bg-gray-100" />
                    <label htmlFor="active" className="ml-2 block text-sm text-gray-900">
                      Menor que 5 anos
                    </label>
                  </div>
                </div>
              )
            }
          </div>

          <div className="flex flex-wrap -mx-3 mb-2 justify-between">
            <div
              onClick={() => {
                setIsAdvancedSearch(!isAdvancedSearch)
              }}
              className="flex ml-2 items-center space-x-1 hover:opacity-90 cursor-pointer transition-all"
            >
              {isAdvancedSearch
                ? (
                  <MinusCircle size={24} weight="light" className="text-gray-800" />
                ) : (
                  <PlusCircle size={24} weight="light" className="text-gray-800" />
                )
              }
              <span className="text-gray-800">Busca avançada</span>
            </div>
            <div className="px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleClear} type="button">Limpar</button>
              <button className="btn_success" type="submit">Buscar</button>
            </div>
          </div>
        </form>

        <div className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h3 className="text-lg md:text-xl">Resultado da busca</h3>
          </div>

          {patients.length > 0
            ? (
              <table className="w-full text-sm text-left text-gray-500 dark:text-gray-400">
                <thead className="sticky top-0 text-xs text-gray-700 uppercase bg-gray-50 dark:bg-gray-700 dark:text-gray-400">
                  <tr>
                    <th scope="col" className="py-3 px-6">
                      CPF
                    </th>
                    <th scope="col" className="py-3 px-6">
                      Nome
                    </th>
                    <th scope="col" className="py-3 px-6">
                      Microárea
                    </th>
                    <th scope="col" className="py-3 px-6">
                      Idade
                    </th>
                    <th scope="col" className="py-3 px-6">
                      Ações
                    </th>
                  </tr>
                </thead>
                <tbody className="overflow-x-auto ">
                  {currentItems?.map((patient) => {
                    return (
                      <tr key={patient.id} className="bg-white border-b dark:bg-gray-800 dark:border-gray-700 hover:bg-gray-50 dark:hover:bg-gray-600">
                        <td scope="row" className="py-4 px-6 font-medium text-gray-900 whitespace-nowrap dark:text-white">
                          {CPFMaskToView(patient.cpf)}
                        </td>
                        <td className="py-4 px-6">
                          {patient.name}
                        </td>
                        <td className="py-4 px-6">
                          {patient.microarea}
                        </td>
                        <td className="py-4 px-6">
                          {calculateAge(patient.birthdate)}
                        </td>
                        <td className="py-4 px-6">
                          <div className="w-full px-3 flex space-x-2">
                            <button className="btn_success" onClick={() => handleUpdatePatient(patient.id)}>Alterar</button>
                          </div>
                        </td>
                      </tr>
                    );
                  })}
                </tbody>
              </table>
            )
            : (
              <div className="w-full flex items-center justify-center">
                <span className="text-lg md:text-xl">Nenhum paciente encontrado!</span>
              </div>
            )
          }


          {patients.length > itemsPerPage && (
            <nav className="flex justify-between items-center pt-4" aria-label="Table navigation">
              <span className="text-sm font-normal text-gray-500 dark:text-gray-400">Listando <span className="font-semibold text-gray-900 dark:text-white">{itemOffset === 0 ? 1 : itemOffset + 1} - {(itemsPerPage + itemOffset) > patients.length ? patients.length : (itemsPerPage + itemOffset)}</span> de <span className="font-semibold text-gray-900 dark:text-white">{patients.length}</span></span>
              <div className="inline-flex items-center -space-x-px">
                <ReactPaginate
                  pageCount={pageCount}
                  onPageChange={handlePageClick}
                  pageRangeDisplayed={2}
                  marginPagesDisplayed={1}
                  nextLabel={<CaretRight size={20} />}
                  previousLabel={<CaretLeft size={20} />}
                  breakLabel={<DotsThree size={32} />}
                  containerClassName="inline-flex"
                  activeClassName="bg-indigo-600"
                  activeLinkClassName="text-white hover:bg-indigo-600 bg-indigo-600"
                  pageClassName="bg-white text-indigo-600 hover:bg-indigo-100 transition-colors"
                  pageLinkClassName="h-10 w-10 p-2  duration-150 border border-r-0 border-indigo-600 focus:shadow-outline  flex items-center justify-center"
                  previousLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 rounded-l-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  nextLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-indigo-600 rounded-r-lg focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                  breakLinkClassName="h-10 w-10 p-2 text-indigo-600 transition-colors duration-150 bg-white border border-r-0 border-indigo-600 focus:shadow-outline hover:bg-indigo-100 flex items-center justify-center"
                />
              </div>
            </nav>
          )
          }

          <div className="mt-3 flex w-full items-center justify-end">
            <button onClick={handleGeneratePDF} className="btn_success flex items-center justify-center space-x-2"><span>Gerar</span> <FilePdf size={24} weight="light" /></button>
          </div>
        </div>
      </main>
      <Toaster />
    </div>
  );
}

export { SearchPatients };