import { useCallback, useEffect, useState } from "react";
import { useNavigate, useParams } from "react-router-dom";

/* libs */
import toast, { Toaster } from "react-hot-toast";
import { AxiosError } from "axios";
import { MapContainer, Marker, TileLayer, ZoomControl } from "react-leaflet";
import MultiSelect, { ActionMeta, MultiValue } from "react-select";
import { format } from "date-fns";
import * as yup from 'yup';

/* components */
import { Sidebar } from "../../../components/Sidebar";
import { Input } from "../../../components/Input";
import { Select } from "../../../components/Select";
import { Textarea } from "../../../components/Textarea";

/* aproveitamento da página de criação */
import { MapEvent, MapEventFlyTo, MapEventPolygon, PatientForm, PatientFormErrors, patientSchema } from "../CreatePatients";

/* utils */
import { api } from "../../../api/axios";
import MapIcon from '../../../utils/mapIcon';
import { transformJSONToCoordsArray } from "../../../utils/transformJSONToCoordsArray";
import { multiSelectStyles } from "../../../utils/multiSelectStyles";
import { CPFMaskToView } from "../../../utils/masks";
import { useAuth } from "../../../context/AuthContext";

interface PatientResponse {
  id: string,
  name: string,
  cpf: string,
  birthdate: string,
  sex: "FEMININO" | "MASCULINO",
  observation: null | string,
  address: {
    number: number,
    district: string,
    complement: string | null,
    publicPlace: string
  },
  localization: {
    latitude: number,
    longitude: number
  },
  conditions: {
    id: string,
    abbr: string
  }[],
  microareaId: string,
}

interface Coords {
  latitude: number,
  longitude: number,
}

interface MicroareaResponse {
  id: string;
  number: number;
  createdAt: string;
  deletedAt: string | null;
  localization: Coords[];
}

interface ConditionResponse {
  id: string;
  abbr: string;
  description: string;
  deletedAt: string | null;
}

const optionsSexSelect = [
  { key: "FEMININO", value: "Feminino" },
  { key: "MASCULINO", value: "Masculino" },
];

/* Dourados - MS*/
const initialPosition = {
  lat: -22.2307329,
  lng: -54.8062403
}



function UpdatePatients() {
  const { user } = useAuth();

  const navigate = useNavigate();
  const { id } = useParams();

  const [isOpen, setIsOpen] = useState(false);
  const [patientErrors, setPatientErrors] = useState<PatientFormErrors>({
    cpf: null,
    name: null,
    sex: null,
    birthdate: null,
    publicPlace: null,
    number: null,
    district: null,
    microareaId: null
  } as PatientFormErrors);

  const [position, setPosition] = useState({ latitude: 0, longitude: 0 });

  const [microareas, setMicroareas] = useState<MicroareaResponse[]>([]);
  const [microareaSelected, setMicroareaSelected] = useState<{ coords: L.LatLngExpression[], microareaNumber: number }>({} as { coords: L.LatLngExpression[], microareaNumber: number });

  const [patient, setPatient] = useState<PatientForm>({} as PatientForm);
  const [patientId, setPatientId] = useState<string>('');

  const [optionsMicroareaSelect, setOptionsMicroareaSelect] = useState<{ key: string; value: string; }[]>([]);
  const [optionsConditionsSelect, setOptionsConditionsSelect] = useState<{ value: string; label: string; }[]>([]);

  /* Busca Paciente a ser editado quando a página está preste a carregar*/
  useEffect(() => {
    const getPatient = async () => {
      try {
        const response = await api.get<PatientResponse>(`/patients/${id}`);

        if (response.data) {
          const patient = response.data;
          const patientForm = {
            name: patient.name,
            cpf: patient.cpf,
            birthdate: format(new Date(patient.birthdate), 'yyyy-MM-dd'),
            sex: patient.sex,
            observation: patient.observation,
            publicPlace: patient.address.publicPlace,
            number: patient.address.number,
            complement: patient.address.complement,
            district: patient.address.district,
            microareaId: patient.microareaId,
            conditions: patient.conditions.map(condition => condition.id),
            position: {
              latitude: patient.localization.latitude,
              longitude: patient.localization.longitude
            }
          } as PatientForm;

          setPatient(patientForm);
          setPosition(patientForm.position);
          setPatientId(patient.id);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }

    getPatient();
  }, []);

  /* Buscar todas as microáreas já criadas */
  useEffect(() => {
    const listMicroareas = async () => {
      try {
        const response = await api.get<MicroareaResponse[]>('/microareas');

        if (response.data) {
          setMicroareas(response.data);

          const options = response.data.map(microarea => {
            return {
              key: microarea.id,
              value: (microarea.number).toString()
            }
          });

          setOptionsMicroareaSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user) {
      listMicroareas();
    }
  }, [user]);

  /* Busca todas as condições de saúde existentes */
  useEffect(() => {
    const listConditons = async () => {
      try {
        const response = await api.get<ConditionResponse[]>('/conditions');

        if (response.data) {
          let options = [] as { value: string; label: string; }[];
          response.data.forEach((condition) => {
            if (condition.deletedAt === null) { // Verifica se a condição está ativa
              options.push({
                value: condition.id,
                label: condition.abbr
              });
            }
          })
          setOptionsConditionsSelect(options);
        }
      } catch (error) {
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }
    }
    if (user)
      listConditons();
  }, [user]);

  useEffect(() => {
    if (patient.microareaId && microareas.length > 0) {
      const microarea = microareas.find(microarea => microarea.id === patient.microareaId);

      if (microarea) {
        setMicroareaSelected((prev) => {
          return {
            ...prev,
            coords: transformJSONToCoordsArray(microarea.localization),
            microareaNumber: microarea.number
          }
        });
      }
    }
  }, [patient, microareas]);

  /* Abre e Fecha sidebar no responsivo */
  const handleOpenSidebar = () => {
    setIsOpen(!isOpen);
  }

  /* Atualiza no estado de Paciente os campos do formulário */
  const handleChange = useCallback((event: React.FormEvent<HTMLInputElement> | React.FormEvent<HTMLTextAreaElement>) => {
    setPatient({
      ...patient,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (!!event.currentTarget.name) {
      let key = event.currentTarget.name;
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }, [patient]);

  /* Atualiza o estado para input do tipo select simples */
  const handleSelectSingle = (event: React.FormEvent<HTMLSelectElement>) => {
    setPatient({
      ...patient,
      [event.currentTarget.name]: event.currentTarget.value,
    });

    if (event.currentTarget.name === "microareaId") {
      const microarea = microareas.find(microarea => microarea.id === event.currentTarget.value);
      if (microarea) {
        setMicroareaSelected((prev) => {
          return {
            ...prev,
            coords: transformJSONToCoordsArray(microarea.localization),
            microareaNumber: microarea.number
          }
        });
      }
    }

    if (!!event.currentTarget.name && event.currentTarget.value !== 'default') {
      let key = event.currentTarget.name;
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  };

  /* Atualiza o estado para input do tipo select multiplo */
  const handleSelectMulti = (newValue: MultiValue<{ value: string, label: string }>, actionMeta: ActionMeta<any>) => {
    const conditionsCurrent = newValue.map(condition => condition.value);
    setPatient({
      ...patient,
      conditions: conditionsCurrent,
    });

    if (!!newValue && newValue.length > 0) {
      let key = "conditions";
      setPatientErrors((prev) => {
        return {
          ...prev,
          [key]: null
        }
      });
    }
  }

  /* Cancelar Atualização de paciente e retornar para a página de listagem */
  const handleCancel = () => {
    navigate('/patients/list');
  }

  /* Salva os dados alterados do paciente */
  const handleSubmit = async (event: React.FormEvent) => {
    event.preventDefault();

    let isValid = true;

    try {
      /* Validando os Selects */
      if (!patient.sex || patient.sex === 'default') {
        setPatientErrors((prevState) => {
          return {
            ...prevState,
            sex: "Este campo é obrigatório"
          }
        });
        isValid = false;
      }

      if (!patient.microareaId || patient.microareaId === 'default') {
        setPatientErrors((prevState) => {
          return {
            ...prevState,
            microareaId: "Este campo é obrigatório"
          }
        });
        isValid = false;
      }

      /* Validando os demais campos usando o Yup */
      const dataValidateByYup = await patientSchema.validate({
        name: patient.name,
        cpf: patient.cpf.replace(/(\.|\-)/g, ''),
        birthdate: patient.birthdate,
        publicPlace: patient.publicPlace,
        number: patient.number,
        district: patient.district,
      }, {
        abortEarly: false
      });


      if (!isValid) {
        return;
      }

      const data = {
        name: dataValidateByYup.name,
        cpf: dataValidateByYup.cpf.replace(/(\.|\-)/g, ''),
        sex: patient.sex,
        birthdate: format(dataValidateByYup.birthdate, "yyyy-MM-dd"),
        observation: patient.observation || "",
        conditions: patient.conditions,
        address: {
          publicPlace: dataValidateByYup.publicPlace,
          number: dataValidateByYup.number,
          district: dataValidateByYup.district,
          complement: patient.complement || "",
        },
        microareaId: patient.microareaId,
        localization: {
          lat: position.latitude,
          lng: position.longitude
        },
      };

      try {
        const response = await api.put(`/patients/${patientId}`, data);

        if (response.status === 200) {
          toast.success("Paciente atualizado com sucesso!", {
            duration: 6000,
            position: 'top-center'
          });
          navigate('/patients/list');
        }

      } catch (error) {
        /* Erro emitido pelo servidor */
        if (error instanceof AxiosError && error.code === "ERR_BAD_REQUEST") {
          if (error.response?.status === 404 || error.response?.status === 400 || error.response?.status === 401) {
            toast.error(error.response.data.message, {
              duration: 4000,
              position: 'top-center'
            })
          }
        }
      }

    } catch (err) {
      /* Erro na validação dos campos pelo yup */
      if (err instanceof yup.ValidationError) {
        err.inner.forEach(error => {
          if (error.path) {
            setPatientErrors((prevState) => {
              return {
                ...prevState,
                [error.path as string]: error.message
              }
            });
          }
        });
      }
    }
  }

  return (
    <div className="w-full flex">
      <div className="flex flex-nowrap min-h-screen flex-row md:fixed">
        <Sidebar handleOpenSidebar={handleOpenSidebar} isOpen={isOpen} setIsOpen={setIsOpen} />
      </div>
      <main className={isOpen ? `flex-1 relative flex-col hidden mm:flex mm:ml-[250px]` : `flex-1 flex relative flex-col md:ml-[250px]`}>
        {isOpen && <div className="bg-zinc-700 opacity-80 absolute w-full h-full z-40 md:hidden"></div>
        }
        <div className="h-20 w-full mb-9 flex justify-center items-center">
          <h2 className="text-xl md:text-3xl">Editar Paciente - {patient?.name}</h2>
        </div>

        <form onSubmit={handleSubmit} className="w-full px-5 lg:px-20 bg-gray-50 py-6 lg:max-w-[80%] mx-auto">
          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Informações Gerais</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Nome Completo"
                id="name"
                type="text"
                name="name"
                placeholder="Nome completo"
                error={patientErrors.name ? patientErrors.name : ''}
                onChange={handleChange}
                value={patient.name || ''}
              />
            </div>
          </div>
          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="CPF"
                id="cpf"
                type="text"
                name="cpf"
                mask="CPF"
                placeholder="999.999.999-99"
                onChange={handleChange}
                error={patientErrors.cpf ? patientErrors.cpf : ''}
                value={patient.cpf ? CPFMaskToView(patient.cpf) : ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Select
                label="Sexo"
                id="sex"
                name="sex"
                onChange={handleSelectSingle}
                options={optionsSexSelect}
                textDefault="Selecione um sexo"
                error={patientErrors.sex ? patientErrors.sex : ''}
                value={patient.sex || 'default'}
              />
            </div>
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Input
                label="Data de Nascimento"
                id="birthdate"
                type="date"
                name="birthdate"
                onChange={handleChange}
                error={patientErrors.birthdate ? patientErrors.birthdate : ''}
                value={patient.birthdate || ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Textarea
                label="Observação"
                id="observation"
                name="observation"
                placeholder="Digite uma observação sobre o paciente"
                onChange={handleChange}
                value={patient.observation || ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <label className="block mb-1" htmlFor="conditions">Condição de Saúde</label>
              <MultiSelect
                className="w-full"
                options={optionsConditionsSelect}
                isMulti
                id="conditions"
                name="conditions"
                styles={multiSelectStyles}
                placeholder="Selecione as condições de saúde do paciente"
                noOptionsMessage={() => "Não há condições de saúde"}
                onChange={handleSelectMulti}
                value={patient.conditions && optionsConditionsSelect.filter(option => patient.conditions.includes(option.value))}
              />
            </div>
          </div>

          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Endereço</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6 md:flex-row space-y-6 md:space-y-0">
            <div className="w-full md:w-auto md:flex-1 px-3">
              <Input
                label="Logradouro"
                id="publicPlace"
                type="text"
                name="publicPlace"
                placeholder="Digite o nome da rua"
                error={patientErrors.publicPlace ? patientErrors.publicPlace : ''}
                onChange={handleChange}
                value={patient.publicPlace || ''}
              />
            </div>
            <div className="w-full md:w-auto px-3">
              <Input
                label="Número"
                id="number"
                type="number"
                name="number"
                placeholder="N°"
                error={patientErrors.number ? patientErrors.number : ''}
                onChange={handleChange}
                value={patient.number || ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Input
                label="Bairro"
                id="district"
                type="text"
                name="district"
                placeholder="Nome do bairro"
                error={patientErrors.district ? patientErrors.district : ''}
                onChange={handleChange}
                value={patient.district || ''}
              />
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Textarea
                label="Complemento"
                id="complement"
                name="complement"
                resize
                placeholder="Digite o complemento"
                onChange={handleChange}
                value={patient.complement || ''}
              />
            </div>
          </div>

          <div className="-mx-2 mb-2 px-5">
            <hr className="border-gray-400" />
          </div>
          <div className="w-full -mx-3 mb-6 flex justify-center items-center">
            <h2 className="text-lg md:text-xl">Geolocalização</h2>
          </div>

          <div className="flex flex-wrap -mx-3 mb-6">
            <div className="w-full px-3">
              <Select
                label="Microárea"
                id="microareaId"
                name="microareaId"
                onChange={handleSelectSingle}
                options={optionsMicroareaSelect}
                textDefault="Selecione uma microárea"
                error={patientErrors.microareaId ? patientErrors.microareaId : ''}
                value={patient.microareaId || 'default'}
              />
            </div>
          </div>

          {/* Mapa para geolocalização */}
          <div className=" -mx-3 mb-6 px-3">
            <span className="block mb-1">Ajuste o marcador para ser mais precisa a localização</span>
            <div className="w-full h-[350px] sm:h-[450px] lg:h-[550px]">
              <MapContainer
                center={position.latitude ? [position.latitude, position.longitude] : initialPosition}
                zoom={14}
                zoomControl={false}
                className="w-full h-full"
              >
                <MapEvent setPosition={setPosition} />
                <MapEventFlyTo latitude={position.latitude} longitude={position.longitude} />
                <MapEventPolygon coords={microareaSelected.coords} />
                <TileLayer
                  attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
                  url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
                />
                <ZoomControl position='bottomright' zoomInTitle='Aumentar' zoomOutTitle='Diminuir' />

                <Marker
                  interactive={false}
                  icon={MapIcon}
                  position={[position.latitude, position.longitude]}
                />
              </MapContainer>
            </div>
          </div>

          <div className="flex flex-wrap -mx-3 mb-2">
            <div className="w-full px-3 flex space-x-2">
              <button className="btn_error text-white" onClick={handleCancel}>Cancelar</button>
              <button className="btn_success">Salvar</button>
            </div>
          </div>
        </form>
      </main>
      <Toaster />
    </div>
  );
}

export { UpdatePatients }